@@ -52,7 +52,7 @@ struct _EphyDownloadPrivate
   EphyDownloadActionType action;
   guint32 start_time;
 
-  GtkWidget *window;
+  GtkWindow *window;
   GtkWidget *widget;
 };
 
@@ -125,7 +125,7 @@ ephy_download_set_property (GObject      *object,
       ephy_download_set_action (download, g_value_get_enum (value));
       break;
     case PROP_WINDOW:
-      ephy_download_set_window (download, g_value_get_object (value));
+      download->priv->window = g_value_dup_object (value);
       break;
     case PROP_WIDGET:
       ephy_download_set_widget (download, g_value_get_object (value));
@@ -152,12 +152,7 @@ ephy_download_set_property (GObject      *object,
 char *
 ephy_download_get_content_type (EphyDownload *download)
 {
-#ifdef HAVE_WEBKIT2
   WebKitURIResponse *response;
-#else
-  WebKitNetworkResponse *response;
-  SoupMessage *message;
-#endif
   char *content_type = NULL;
   GError *error = NULL;
 
@@ -185,17 +180,9 @@ ephy_download_get_content_type (EphyDownload *download)
     return content_type;
 
   /* Fallback to Soup */
-#ifdef HAVE_WEBKIT2
   response = webkit_download_get_response (download->priv->download);
   if (response)
     content_type = g_strdup (webkit_uri_response_get_mime_type (response));
-#else
-  response = webkit_download_get_network_response (download->priv->download);
-  message = webkit_network_response_get_message (response);
-
-  if (message != NULL)
-    content_type = g_strdup (soup_message_headers_get_content_type (message->response_headers, NULL));
-#endif
 
   LOG ("ephy_download_get_content_type: Soup: %s", content_type);
 
@@ -241,7 +228,7 @@ static const char*
 file_is_compressed (const char *filename)
 {
   int i;
-  static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+  static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", ".xz", NULL};
 
   for (i = 0; compression[i] != NULL; i++) {
     if (g_str_has_suffix (filename, compression[i]))
@@ -377,9 +364,6 @@ ephy_download_set_destination_uri (EphyDownload *download,
 
   priv->destination = g_strdup (destination);
 
-#ifndef HAVE_WEBKIT2
-  webkit_download_set_destination_uri (priv->download, priv->destination);
-#endif
   g_object_notify (G_OBJECT (download), "destination");
 }
 
@@ -392,19 +376,7 @@ ephy_download_set_destination_uri (EphyDownload *download,
 void
 ephy_download_set_auto_destination (EphyDownload *download)
 {
-#ifdef HAVE_WEBKIT2
-  /* In WebKit2 priv->destination == NULL means auto_destination */
-#else
-  char *dest;
-
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
-
-  dest = define_destination_uri (download,
-                                 webkit_download_get_suggested_filename (download->priv->download));
-  ephy_download_set_destination_uri (download, dest);
-
-  g_free (dest);
-#endif
+  /* TODO: in WebKit2 priv->destination == NULL means auto_destination, remove this? */
 }
 
 /**
@@ -426,31 +398,6 @@ ephy_download_set_action (EphyDownload *download,
   g_object_notify (G_OBJECT (download), "action");
 }
 
-/**
- * ephy_download_set_window:
- * @download: an #EphyDownload
- * @window: #GtkWidget that produced @download
- *
- * Sets @window to be @download's parent, this means that @download will be
- * shown on @window's #EphyWindow (where it is contained).
- **/
-void
-ephy_download_set_window (EphyDownload *download,
-                          GtkWidget *window)
-{
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
-
-  if (download->priv->window != NULL)
-    g_object_unref (download->priv->window);
-
-  download->priv->window = NULL;
-
-  if (window != NULL)
-    download->priv->window = g_object_ref (window);
-
-  g_object_notify (G_OBJECT (download), "window");
-}
-
 /**
  * ephy_download_set_widget:
  * @download: an #EphyDownload
@@ -511,12 +458,12 @@ ephy_download_get_webkit_download (EphyDownload *download)
  * ephy_download_get_window:
  * @download: an #EphyDownload
  *
- * Gets the window set as the parent of @download, this can be NULL if no
+ * Gets the window set as the parent of @download, this can be %NULL if no
  * specific window generated this download.
  *
- * Returns: (transfer none): a #GtkWidget
+ * Returns: (transfer none): a #GtkWindow
  **/
-GtkWidget *
+GtkWindow *
 ephy_download_get_window (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
@@ -610,9 +557,6 @@ ephy_download_start (EphyDownload *download)
 
   if (priv->destination == NULL)
     ephy_download_set_auto_destination (download);
-#ifndef HAVE_WEBKIT2
-  webkit_download_start (priv->download);
-#endif
 }
 
 /**
@@ -652,11 +596,7 @@ ephy_download_do_download_action (EphyDownload *download,
 
     priv = download->priv;
 
-#ifdef HAVE_WEBKIT2
     destination_uri = webkit_download_get_destination (priv->download);
-#else
-    destination_uri = webkit_download_get_destination_uri (priv->download);
-#endif
     destination = g_file_new_for_uri (destination_uri);
 
     switch ((action ? action : priv->action)) {
@@ -670,7 +610,8 @@ ephy_download_do_download_action (EphyDownload *download,
         break;
       case EPHY_DOWNLOAD_ACTION_OPEN:
         LOG ("ephy_download_do_download_action: open");
-        ret = ephy_file_launch_handler (NULL, destination, priv->start_time);
+        ret = ephy_embed_shell_launch_handler (ephy_embed_shell_get_default (), 
+                                               destination, NULL, priv->start_time);
         break;
       case EPHY_DOWNLOAD_ACTION_NONE:
         LOG ("ephy_download_do_download_action: none");
@@ -701,6 +642,7 @@ ephy_download_dispose (GObject *object)
   priv = download->priv;
 
   if (priv->download) {
+    g_signal_handlers_disconnect_matched (priv->download, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, download);
     g_object_unref (priv->download);
     priv->download = NULL;
   }
@@ -715,7 +657,7 @@ ephy_download_dispose (GObject *object)
     priv->widget = NULL;
   }
 
-  ephy_embed_shell_remove_download (embed_shell, download);
+  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
 
   G_OBJECT_CLASS (ephy_download_parent_class)->dispose (object);
 }
@@ -827,7 +769,7 @@ ephy_download_class_init (EphyDownloadClass *klass)
                                                       G_PARAM_STATIC_BLURB));
 
   /**
-   * EphyDownload::window:
+   * EphyDownload:window:
    *
    * Window that produced the download, the download will be shown in its
    * parent window.
@@ -836,8 +778,9 @@ ephy_download_class_init (EphyDownloadClass *klass)
                                    g_param_spec_object ("window",
                                                         "A GtkWindow",
                                                         "Window that produced this download.",
-                                                        GTK_TYPE_WIDGET,
+                                                        GTK_TYPE_WINDOW,
                                                         G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
@@ -904,7 +847,6 @@ ephy_download_init (EphyDownload *download)
   download->priv->widget = NULL;
 }
 
-#ifdef HAVE_WEBKIT2
 static gboolean
 download_decide_destination_cb (WebKitDownload *wk_download,
                                 const gchar *suggested_filename,
@@ -955,41 +897,9 @@ download_finished_cb (WebKitDownload *wk_download,
   else
     ephy_download_do_download_action (download, priv->action);
 
-  ephy_embed_shell_remove_download (embed_shell, download);
+  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
 }
-#else
-static void
-download_status_changed_cb (GObject *object,
-                            GParamSpec *pspec,
-                            EphyDownload *download)
-{
-  WebKitDownloadStatus status;
-  EphyDownloadPrivate *priv;
 
-  priv = download->priv;
-
-  status = webkit_download_get_status (priv->download);
-
-  if (status == WEBKIT_DOWNLOAD_STATUS_FINISHED) {
-    g_signal_emit_by_name (download, "completed");
-
-    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_AUTO_DOWNLOADS) &&
-        priv->action == EPHY_DOWNLOAD_ACTION_NONE) {
-      ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_AUTO);
-    } else {
-      ephy_download_do_download_action (download, priv->action);
-    }
-
-    ephy_embed_shell_remove_download (embed_shell, download);
-  } else if (status == WEBKIT_DOWNLOAD_STATUS_CANCELLED ||
-             status == WEBKIT_DOWNLOAD_STATUS_ERROR) {
-  } else if (status == WEBKIT_DOWNLOAD_STATUS_STARTED) {
-    ephy_embed_shell_add_download (embed_shell, download);
-  }
-}
-#endif
-
-#ifdef HAVE_WEBKIT2
 static void
 download_failed_cb (WebKitDownload *wk_download,
                     GError *error,
@@ -1002,29 +912,10 @@ download_failed_cb (WebKitDownload *wk_download,
   LOG ("error (%d - %d)! %s", error->code, 0, error->message);
   g_signal_emit_by_name (download, "error", 0, error->code, error->message, &ret);
 }
-#else
-static gboolean
-download_error_cb (WebKitDownload *download,
-                   gint error_code,
-                   gint error_detail,
-                   char *reason,
-                   gpointer user_data)
-{
-  EphyDownload *ephy_download;
-  gboolean ret = FALSE;
-
-  ephy_download = EPHY_DOWNLOAD (user_data);
-
-  LOG ("error (%d - %d)! %s", error_code, error_detail, reason);
-  g_signal_emit_by_name (ephy_download, "error",
-                         error_code, error_detail, reason, &ret);
-
-  return ret;
-}
-#endif
 
 /**
  * ephy_download_new:
+ * @parent: the #GtkWindow parent of the download, or %NULL
  *
  * Creates a new #EphyDownload. You can use ephy_download_new_for_download and
  * ephy_download_new_for_uri as convenience functions to create #EphyDownload
@@ -1033,32 +924,31 @@ download_error_cb (WebKitDownload *download,
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new (void)
+ephy_download_new (GtkWindow *parent)
 {
-  return g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
+  return g_object_new (EPHY_TYPE_DOWNLOAD, "window", parent, NULL);
 }
 
 /**
  * ephy_download_new_for_download:
  * @download: a #WebKitDownload to wrap
+ * @parent: the #GtkWindow parent of the download, or %NULL
  *
  * Wraps @download in an #EphyDownload.
  *
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new_for_download (WebKitDownload *download)
+ephy_download_new_for_download (WebKitDownload *download,
+                                GtkWindow *parent)
 {
   EphyDownload *ephy_download;
-#ifdef HAVE_WEBKIT2
   WebKitURIRequest *request;
-#endif
 
   g_return_val_if_fail (WEBKIT_IS_DOWNLOAD (download), NULL);
 
-  ephy_download = ephy_download_new ();
+  ephy_download = ephy_download_new (parent);
 
-#ifdef HAVE_WEBKIT2
   g_signal_connect (download, "decide-destination",
                     G_CALLBACK (download_decide_destination_cb),
                     ephy_download);
@@ -1071,27 +961,14 @@ ephy_download_new_for_download (WebKitDownload *download)
   g_signal_connect (download, "failed",
                     G_CALLBACK (download_failed_cb),
                     ephy_download);
-#else
-  g_signal_connect (download, "notify::status",
-                    G_CALLBACK (download_status_changed_cb),
-                    ephy_download);
-  g_signal_connect (download, "error",
-                    G_CALLBACK (download_error_cb),
-                    ephy_download);
-#endif
 
   ephy_download->priv->download = g_object_ref (download);
-#ifdef HAVE_WEBKIT2
+  g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
   request = webkit_download_get_request (download);
   ephy_download->priv->source = g_strdup (webkit_uri_request_get_uri (request));
-#else
-  ephy_download->priv->source = g_strdup (webkit_download_get_uri (download));
-#endif
 
-#ifdef HAVE_WEBKIT2
   /* In WebKit2 the download has already started */
-  ephy_embed_shell_add_download (embed_shell, ephy_download);
-#endif
+  ephy_embed_shell_add_download (ephy_embed_shell_get_default (), ephy_download);
 
   return ephy_download;
 }
@@ -1099,33 +976,24 @@ ephy_download_new_for_download (WebKitDownload *download)
 /**
  * ephy_download_new_for_uri:
  * @uri: a source URI from where to download
+ * @parent: the #GtkWindow parent of the download, or %NULL
  *
  * Creates an #EphyDownload to download @uri.
  *
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new_for_uri (const char *uri)
+ephy_download_new_for_uri (const char *uri,
+                           GtkWindow *parent)
 {
   EphyDownload *ephy_download;
   WebKitDownload *download;
-#ifndef HAVE_WEBKIT2
-  WebKitNetworkRequest *request;
-#endif
 
   g_return_val_if_fail (uri != NULL, NULL);
 
-#ifdef HAVE_WEBKIT2
   download = webkit_web_context_download_uri (webkit_web_context_get_default (), uri);
-#else
-  request = webkit_network_request_new (uri);
-  download = webkit_download_new (request);
 
-  g_return_val_if_fail (download != NULL, NULL);
-  g_object_unref (request);
-#endif
-
-  ephy_download = ephy_download_new_for_download (download);
+  ephy_download = ephy_download_new_for_download (download, parent);
   g_object_unref (download);
 
   return ephy_download;
