@@ -29,7 +29,8 @@
 #include "ephy-settings.h"
 
 #include <glib.h>
-#include <webkit/webkit.h>
+#include <math.h>
+#include <webkit2/webkit2.h>
 
 typedef struct
 {
@@ -39,41 +40,81 @@ typedef struct
   void (*callback) (GSettings *settings, char *key, gpointer data);
 } PrefData;
 
-static WebKitWebSettings *webkit_settings = NULL;
+#define ENABLE_SCRIPTS_SETTING "enable-javascript"
+#define DEFAULT_ENCODING_SETTING "default-charset"
+static WebKitSettings *webkit_settings = NULL;
+static WebKitWebViewGroup *web_view_group = NULL;
+
+static void
+user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
+                                          GAsyncResult *result,
+                                          gpointer user_data)
+{
+  gssize bytes;
+
+  bytes = g_output_stream_splice_finish (output_stream, result, NULL);
+  if (bytes > 0) {
+    webkit_web_view_group_add_user_style_sheet (web_view_group,
+                                                g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
+                                                NULL, NULL, NULL, WEBKIT_INJECTED_CONTENT_FRAMES_ALL);
+  }
+}
+
+static void
+user_style_seet_read_cb (GFile *file,
+                         GAsyncResult *result,
+                         gpointer user_data)
+{
+  GFileInputStream *input_stream;
+  GOutputStream *output_stream;
+
+  input_stream = g_file_read_finish (file, result, NULL);
+  if (!input_stream)
+    return;
+
+  output_stream = g_memory_output_stream_new_resizable ();
+  g_output_stream_splice_async (output_stream, G_INPUT_STREAM (input_stream),
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                G_PRIORITY_DEFAULT,
+                                NULL,
+                                (GAsyncReadyCallback)user_style_sheet_output_stream_splice_cb,
+                                NULL);
+  g_object_unref (input_stream);
+  g_object_unref (output_stream);
+}
 
 static void
 webkit_pref_callback_user_stylesheet (GSettings *settings,
                                       char *key,
                                       gpointer data)
 {
-  gboolean value = FALSE;
-  char *uri = NULL;
-  char *webkit_pref = data;
+  gboolean value;
 
   value = g_settings_get_boolean (settings, key);
 
-  if (value)
-    /* We need the leading file://, so use g_strconcat instead
-     * of g_build_filename */
-    uri = g_strconcat ("file://",
-                       ephy_dot_dir (),
-                       G_DIR_SEPARATOR_S,
-                       USER_STYLESHEET_FILENAME,
-                       NULL);
+  if (!value)
+    webkit_web_view_group_remove_all_user_style_sheets (web_view_group);
+  else {
+    GFile *file;
+    char *filename;
 
-  g_object_set (webkit_settings, webkit_pref, uri, NULL);
-  g_free (uri);
+    filename = g_build_filename (ephy_dot_dir (), USER_STYLESHEET_FILENAME, NULL);
+    file = g_file_new_for_path (filename);
+    g_free (filename);
+
+    g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
+                       (GAsyncReadyCallback)user_style_seet_read_cb, NULL);
+    g_object_unref (file);
+  }
 }
 
 static char *
-webkit_pref_get_internal_user_agent (void)
+webkit_pref_get_vendor_user_agent (void)
 {
-  char *user_agent;
-  char *webkit_user_agent;
-  char *vendor_user_agent;
+  char *vendor_user_agent = NULL;
   GKeyFile *branding_keyfile;
 
-  vendor_user_agent = NULL;
   branding_keyfile = g_key_file_new ();
 
   if (g_key_file_load_from_file (branding_keyfile, SHARE_DIR"/branding.conf",
@@ -106,16 +147,30 @@ webkit_pref_get_internal_user_agent (void)
 
   g_key_file_free (branding_keyfile);
 
-  g_object_get (webkit_settings, "user-agent", &webkit_user_agent, NULL);
+  return vendor_user_agent;
+}
 
-  user_agent = g_strconcat (webkit_user_agent, " ",
-                            vendor_user_agent ? vendor_user_agent : "",
-                            vendor_user_agent ? " " : "",
-                            "Epiphany/"VERSION,
-                            NULL);
+static const char *
+webkit_pref_get_internal_user_agent (void)
+{
+  static char *user_agent = NULL;
+  static gboolean initialized = FALSE;
+  char *vendor_user_agent;
+  const char *webkit_user_agent;
 
+  if (initialized)
+    return user_agent;
+
+  initialized = TRUE;
+
+  vendor_user_agent = webkit_pref_get_vendor_user_agent ();
+  if (!vendor_user_agent)
+    return NULL;
+
+  webkit_user_agent = webkit_settings_get_user_agent (webkit_settings);
+  user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
+                                vendor_user_agent, VERSION);
   g_free (vendor_user_agent);
-  g_free (webkit_user_agent);
 
   return user_agent;
 }
@@ -125,21 +180,59 @@ webkit_pref_callback_user_agent (GSettings *settings,
                                  char *key,
                                  gpointer data)
 {
-  char *value = NULL;
-  static char *internal_user_agent = NULL;
-  char *webkit_pref = data;
+  char *value;
+  const char *internal_user_agent;
 
   value = g_settings_get_string (settings, key);
-
-  if (value == NULL || value[0] == '\0') {
-    if (internal_user_agent == NULL)
-      internal_user_agent = webkit_pref_get_internal_user_agent ();
-
-    g_object_set (webkit_settings, webkit_pref, internal_user_agent, NULL);
-  } else
-    g_object_set (webkit_settings, webkit_pref, value, NULL);
-
+  if (value != NULL && value[0] != '\0') {
+    webkit_settings_set_user_agent (webkit_settings, value);
+    g_free (value);
+    return;
+  }
   g_free (value);
+
+  internal_user_agent = webkit_pref_get_internal_user_agent ();
+  if (internal_user_agent)
+    webkit_settings_set_user_agent (webkit_settings, internal_user_agent);
+  else
+    webkit_settings_set_user_agent_with_application_details (webkit_settings,
+                                                             "Epiphany", VERSION);
+}
+
+/* This doesn't contain WebKit2 specific API, but it's only used inside
+ * HAVE_WEBKIT2 blocks, so it gives a compile warning when building
+ * with WebKit1.
+ */
+static gdouble
+get_screen_dpi (GdkScreen *screen)
+{
+  gdouble dpi;
+  gdouble dp, di;
+
+  dpi = gdk_screen_get_resolution (screen);
+  if (dpi != -1)
+    return dpi;
+
+  dp = hypot (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
+  di = hypot (gdk_screen_get_width_mm (screen), gdk_screen_get_height_mm (screen)) / 25.4;
+
+  return dp / di;
+}
+
+static guint
+normalize_font_size (gdouble font_size)
+{
+  /* WebKit2 uses font sizes in pixels. */
+  GdkScreen *screen;
+  gdouble dpi;
+
+  /* FIXME: We should use the view screen instead of the detault one
+   * but we don't have access to the view here.
+   */
+  screen = gdk_screen_get_default ();
+  dpi = screen ? get_screen_dpi (screen) : 96;
+
+  return font_size / 72.0 * dpi;
 }
 
 static void
@@ -176,7 +269,7 @@ webkit_pref_callback_font_size (GSettings *settings,
     pango_font_description_free (desc);
   }
 
-  g_object_set (webkit_settings, webkit_pref, size, NULL);
+  g_object_set (webkit_settings, webkit_pref, normalize_font_size (size), NULL);
   g_free (value);
 }
 
@@ -216,49 +309,6 @@ webkit_pref_callback_font_family (GSettings *settings,
   g_free (value);
 }
 
-/* Part of this code taken from libsoup (soup-session.c) */
-static gchar *
-build_accept_languages_header (GArray *languages)
-{
-  gchar **langs = NULL;
-  gchar *langs_str = NULL;
-  gint delta;
-  gint i;
-
-  g_return_val_if_fail (languages != NULL, NULL);
-
-  /* Calculate deltas for the quality values */
-  if (languages->len < 10)
-    delta = 10;
-  else if (languages->len < 20)
-    delta = 5;
-  else
-    delta = 1;
-
-  /* Set quality values for each language */
-  langs = (gchar **) languages->data;
-  for (i = 0; langs[i] != NULL; i++) {
-    gchar *lang = (gchar *) langs[i];
-    gint quality = 100 - i * delta;
-
-    if (quality > 0 && quality < 100) {
-      gchar buf[8];
-      g_ascii_formatd (buf, 8, "%.2f", quality/100.0);
-      langs[i] = g_strdup_printf ("%s;q=%s", lang, buf);
-    } else {
-      /* Just dup the string in this case */
-      langs[i] = g_strdup (lang);
-    }
-    g_free (lang);
-  }
-
-  /* Get the result string */
-  if (languages->len > 0)
-    langs_str = g_strjoinv (", ", langs);
-
-  return langs_str;
-}
-
 /* Based on Christian Persch's code from gecko backend of epiphany
    (old transform_accept_languages_list() function) */
 static void
@@ -266,15 +316,10 @@ webkit_pref_callback_accept_languages (GSettings *settings,
                                        char *key,
                                        gpointer data)
 {
-  SoupSession *session;
   GArray *array;
   char **languages;
-  char *langs_str;
-  char *webkit_pref;
   int i;
 
-  webkit_pref = data;
-
   languages = g_settings_get_strv (settings, key);
 
   array = g_array_new (TRUE, FALSE, sizeof (char *));
@@ -290,38 +335,32 @@ webkit_pref_callback_accept_languages (GSettings *settings,
 
   ephy_langs_sanitise (array);
 
-  langs_str = build_accept_languages_header (array);
-
-  /* Update Soup session */
-  session = webkit_get_default_session ();
-  g_object_set (G_OBJECT (session), webkit_pref, langs_str, NULL);
+  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (),
+                                              (const char * const *)array->data);
 
   g_strfreev (languages);
-  g_free (langs_str);
   g_array_free (array, TRUE);
 }
 
-void
-ephy_embed_prefs_set_cookie_jar_policy (SoupCookieJar *jar,
-                                        const char *settings_policy)
-{
-  SoupCookieJarAcceptPolicy policy;
 
-  g_return_if_fail (SOUP_IS_COOKIE_JAR (jar));
-  g_return_if_fail (settings_policy != NULL);
+void
+ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
+                                           const char *settings_policy)
+{
+  WebKitCookieAcceptPolicy policy;
 
   if (g_str_equal (settings_policy, "never"))
-    policy = SOUP_COOKIE_JAR_ACCEPT_NEVER;
+    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NEVER;
   else if (g_str_equal (settings_policy, "always"))
-    policy = SOUP_COOKIE_JAR_ACCEPT_ALWAYS;
+    policy = WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS;
   else if (g_str_equal (settings_policy, "no-third-party"))
-    policy = SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY;
+    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY;
   else {
     g_warn_if_reached ();
     return;
   }
 
-  g_object_set (G_OBJECT (jar), SOUP_COOKIE_JAR_ACCEPT_POLICY, policy, NULL);
+  webkit_cookie_manager_set_accept_policy (cookie_manager, policy);
 }
 
 static void
@@ -329,24 +368,15 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
                                            char *key,
                                            gpointer data)
 {
-  SoupSession *session;
-  char *value = NULL;
+  WebKitCookieManager *cookie_manager;
+  char *value;
 
   value = g_settings_get_string (settings, key);
+  if (!value)
+    return;
 
-  if (value) {
-    SoupSessionFeature *jar;
-
-    session = webkit_get_default_session ();
-    jar = soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR);
-    if (!jar) {
-      g_free (value);
-      return;
-    }
-    
-    ephy_embed_prefs_set_cookie_jar_policy (SOUP_COOKIE_JAR (jar), value);
-  }
-
+  cookie_manager = webkit_web_context_get_cookie_manager (webkit_web_context_get_default ());
+  ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, value);
   g_free (value);
 }
 
@@ -362,8 +392,8 @@ webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
                   "default-font-family", "serif",
                   "sans-serif-font-family", "sans-serif",
                   "monospace-font-family", "monospace",
-                  "default-font-size", 12,
-                  "default-monospace-font-size", 10,
+                  "default-font-size", normalize_font_size (12),
+                  "default-monospace-font-size", normalize_font_size (10),
                   NULL);
   } else {
     /* Sync with Epiphany values */
@@ -385,22 +415,30 @@ webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
   }
 }
 
-static void
-replace_system_language_with_concrete_language_string (char **languages)
+static char **
+normalize_languages (char **languages)
 {
   int i;
+  GPtrArray *langs;
 
-  for (i = 0; i < g_strv_length (languages); i++) {
+  langs = g_ptr_array_new ();
+
+  for (i = 0; languages && languages[i]; i++) {
     if (g_str_equal (languages[i], "system")) {
-      char **sys_langs;
+      char **sys_langs = ephy_langs_get_languages ();
+      int j;
 
-      g_free (languages[i]);
-      sys_langs = ephy_langs_get_languages ();
-      languages[i] = g_strjoinv (", ", sys_langs);
+      for (j = 0; sys_langs && sys_langs[j]; j++)
+        g_ptr_array_add (langs, g_strdelimit (g_strdup (sys_langs[i]), "-", '_'));
 
       g_strfreev (sys_langs);
-    }
+    } else
+      g_ptr_array_add (langs, g_strdelimit (g_strdup (languages[i]), "-", '_'));
   }
+
+  g_ptr_array_add (langs, NULL);
+
+  return (char **)g_ptr_array_free (langs, FALSE);
 }
 
 static void
@@ -408,6 +446,7 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
                                             char *key,
                                             gpointer data)
 {
+  WebKitWebContext *web_context = NULL;
   gboolean value = FALSE;
   char **languages = NULL;
   char *langs = NULL;
@@ -415,17 +454,22 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
   value = g_settings_get_boolean (settings, key);
 
   if (value) {
+    char **normalized;
+
     languages = g_settings_get_strv (settings, EPHY_PREFS_WEB_LANGUAGE);
-    replace_system_language_with_concrete_language_string (languages);
-    langs = g_strjoinv (",", languages);
-    g_strdelimit (langs, "-", '_');
+    normalized = normalize_languages (languages);
     g_strfreev (languages);
+
+    languages = normalized;
+    langs = g_strjoinv (",", languages);
   }
 
-  g_object_set (webkit_settings, "enable-spell-checking", value, NULL);
-  g_object_set (webkit_settings, "spell-checking-languages", langs, NULL);
+  web_context = webkit_web_context_get_default ();
+  webkit_web_context_set_spell_checking_enabled (web_context, value);
+  webkit_web_context_set_spell_checking_languages (web_context, (const char* const *)languages);
 
   g_free (langs);
+  g_strfreev (languages);
 }
 
 static const PrefData webkit_pref_entries[] =
@@ -484,27 +528,18 @@ static const PrefData webkit_pref_entries[] =
       webkit_pref_callback_cookie_accept_policy },
   };
 
-static void
-ephy_embed_prefs_apply (EphyEmbed *embed, WebKitWebSettings *settings)
-{
-  webkit_web_view_set_settings (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed),
-                                settings);
-}
-
 void
 ephy_embed_prefs_init (void)
 {
   int i;
+  web_view_group = webkit_web_view_group_new ("Ephy WebView Group");
+  webkit_settings = webkit_web_view_group_get_settings (web_view_group);
 
-  webkit_settings = webkit_web_settings_new ();
-
-  /* Hardcoded settings */
   g_object_set (webkit_settings,
-                "enable-default-context-menu", FALSE,
-                "enable-site-specific-quirks", TRUE,
-                "enable-page-cache", TRUE,
                 "enable-developer-extras", TRUE,
                 "enable-fullscreen", TRUE,
+                "enable-site-specific-quirks", TRUE,
+                "enable-dns-prefetching", TRUE,
                 NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
@@ -526,7 +561,7 @@ ephy_embed_prefs_init (void)
 
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_JAVASCRIPT,
-                   webkit_settings, "enable-scripts",
+                   webkit_settings, ENABLE_SCRIPTS_SETTING,
                    G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_MAIN,
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
@@ -546,23 +581,30 @@ ephy_embed_prefs_init (void)
                    G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_DEFAULT_ENCODING,
-                   webkit_settings, "default-encoding",
+                   webkit_settings, DEFAULT_ENCODING_SETTING,
                    G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_WEBGL,
                    webkit_settings, "enable-webgl",
                    G_SETTINGS_BIND_GET);
+  g_settings_bind (EPHY_SETTINGS_WEB,
+                   EPHY_PREFS_WEB_ENABLE_WEBAUDIO,
+                   webkit_settings, "enable-webaudio",
+                   G_SETTINGS_BIND_GET);
+  g_settings_bind (EPHY_SETTINGS_MAIN,
+                   EPHY_PREFS_ENABLE_SMOOTH_SCROLLING,
+                   webkit_settings, "enable-smooth-scrolling",
+                   G_SETTINGS_BIND_GET);
 }
 
 void
 ephy_embed_prefs_shutdown (void)
 {
-  g_object_unref (webkit_settings);
+  g_object_unref (web_view_group);
 }
 
-void
-ephy_embed_prefs_add_embed (EphyEmbed *embed)
+WebKitWebViewGroup *
+ephy_embed_prefs_get_web_view_group (void)
 {
-  ephy_embed_prefs_apply (embed, webkit_settings);
+  return web_view_group;
 }
-
