@@ -22,16 +22,19 @@
 #include <config.h>
 #include "ephy-embed-shell.h"
 
-#include "ephy-adblock-manager.h"
+#include "ephy-about-handler.h"
 #include "ephy-debug.h"
 #include "ephy-download.h"
+#include "ephy-embed-prefs.h"
 #include "ephy-embed-private.h"
-#include "ephy-embed-single.h"
 #include "ephy-embed-type-builtins.h"
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
+#include "ephy-profile-utils.h"
+#include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
+#include "ephy-web-extension.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -39,6 +42,7 @@
 
 #define PAGE_SETUP_FILENAME "page-setup-gtk.ini"
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
+#define NSPLUGINWRAPPER_SETUP "/usr/bin/mozilla-plugin-config"
 
 #define EPHY_EMBED_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellPrivate))
 
@@ -46,14 +50,15 @@ struct _EphyEmbedShellPrivate
 {
   EphyHistoryService *global_history_service;
   GList *downloads;
-  EphyEmbedSingle *embed_single;
   EphyEncodings *encodings;
-  EphyAdBlockManager *adblock_manager;
   GtkPageSetup *page_setup;
   GtkPrintSettings *print_settings;
   EphyEmbedShellMode mode;
   EphyFrecentStore *frecent_store;
-  guint single_initialised : 1;
+  EphyAboutHandler *about_handler;
+  GDBusProxy *web_extension;
+  guint web_extension_watch_name_id;
+  guint web_extension_form_auth_save_signal_id;
 };
 
 enum
@@ -61,6 +66,10 @@ enum
   DOWNLOAD_ADDED,
   DOWNLOAD_REMOVED,
   PREPARE_CLOSE,
+  RESTORED_WINDOW,
+  WEB_VIEW_CREATED,
+  FORM_AUTH_DATA_SAVE_REQUESTED,
+
   LAST_SIGNAL
 };
 
@@ -89,25 +98,104 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->print_settings);
   g_clear_object (&priv->frecent_store);
   g_clear_object (&priv->global_history_service);
-  g_clear_object (&priv->embed_single);
-  g_clear_object (&priv->adblock_manager);
+
+  if (priv->downloads != NULL) {
+    LOG ("Destroying downloads list");
+    g_list_free_full (priv->downloads, (GDestroyNotify)g_object_unref);
+    priv->downloads = NULL;
+  }
+
+  g_clear_object (&priv->about_handler);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
 
 static void
-ephy_embed_shell_finalize (GObject *object)
+web_extension_form_auth_save_requested (GDBusConnection *connection,
+                                        const char *sender_name,
+                                        const char *object_path,
+                                        const char *interface_name,
+                                        const char *signal_name,
+                                        GVariant *parameters,
+                                        EphyEmbedShell *shell)
 {
-  EphyEmbedShellPrivate *priv = EPHY_EMBED_SHELL (object)->priv;
+  guint request_id;
+  guint64 page_id;
+  const char *hostname;
+  const char *username;
 
-  if (priv->downloads != NULL) {
-    LOG ("Destroying downloads list");
-    g_list_foreach (priv->downloads, (GFunc)g_object_unref, NULL);
-    g_list_free (priv->downloads);
-    priv->downloads = NULL;
+  g_variant_get (parameters, "(ut&s&s)", &request_id, &page_id, &hostname, &username);
+  g_signal_emit (shell, signals[FORM_AUTH_DATA_SAVE_REQUESTED], 0,
+                 request_id, page_id, hostname, username);
+}
+
+static void
+web_extension_proxy_created_cb (GDBusProxy *proxy,
+                                GAsyncResult *result,
+                                EphyEmbedShell *shell)
+{
+  GError *error = NULL;
+
+  shell->priv->web_extension = g_dbus_proxy_new_finish (result, &error);
+  if (!shell->priv->web_extension) {
+    g_warning ("Error creating web extension proxy: %s\n", error->message);
+    g_error_free (error);
+  } else {
+    shell->priv->web_extension_form_auth_save_signal_id =
+      g_dbus_connection_signal_subscribe (g_dbus_proxy_get_connection (shell->priv->web_extension),
+                                          g_dbus_proxy_get_name (shell->priv->web_extension),
+                                          EPHY_WEB_EXTENSION_INTERFACE,
+                                         "FormAuthDataSaveConfirmationRequired",
+                                          EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                          NULL,
+                                          G_DBUS_SIGNAL_FLAGS_NONE,
+                                          (GDBusSignalCallback)web_extension_form_auth_save_requested,
+                                          shell,
+                                          NULL);
   }
+}
 
-  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->finalize (object);
+static void
+web_extension_appeared_cb (GDBusConnection *connection,
+                           const gchar *name,
+                           const gchar *name_owner,
+                           EphyEmbedShell *shell)
+{
+  g_dbus_proxy_new (connection,
+                    G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
+                    G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+                    G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                    NULL,
+                    name,
+                    EPHY_WEB_EXTENSION_OBJECT_PATH,
+                    EPHY_WEB_EXTENSION_INTERFACE,
+                    NULL,
+                    (GAsyncReadyCallback)web_extension_proxy_created_cb,
+                    shell);
+}
+
+static void
+web_extension_vanished_cb (GDBusConnection *connection,
+                           const gchar *name,
+                           EphyEmbedShell *shell)
+{
+  g_clear_object (&shell->priv->web_extension);
+}
+
+static void
+ephy_embed_shell_watch_web_extension (EphyEmbedShell *shell)
+{
+  char *service_name;
+
+  service_name = g_strdup_printf ("%s-%u", EPHY_WEB_EXTENSION_SERVICE_NAME, getpid ());
+  shell->priv->web_extension_watch_name_id =
+    g_bus_watch_name (G_BUS_TYPE_SESSION,
+                      service_name,
+                      G_BUS_NAME_WATCHER_FLAGS_NONE,
+                      (GBusNameAppearedCallback) web_extension_appeared_cb,
+                      (GBusNameVanishedCallback) web_extension_vanished_cb,
+                      shell, NULL);
+  g_free (service_name);
 }
 
 /**
@@ -124,7 +212,7 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
   if (shell->priv->global_history_service == NULL) {
     char *filename;
 
-    filename = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+    filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
     shell->priv->global_history_service = ephy_history_service_new (filename);
     g_free (filename);
     g_return_val_if_fail (shell->priv->global_history_service, NULL);
@@ -186,59 +274,6 @@ ephy_embed_shell_get_frecent_store (EphyEmbedShell *shell)
   return shell->priv->frecent_store;
 }
 
-static GObject *
-impl_get_embed_single (EphyEmbedShell *shell)
-{
-  EphyEmbedShellPrivate *priv;
-
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
-
-  priv = shell->priv;
-
-  if (priv->embed_single != NULL &&
-      !priv->single_initialised) {
-    g_warning ("ephy_embed_shell_get_embed_single called while the single is being initialised!\n");
-    return G_OBJECT (priv->embed_single);
-  }
-
-  if (priv->embed_single == NULL) {
-    priv->embed_single = EPHY_EMBED_SINGLE (g_object_new (EPHY_TYPE_EMBED_SINGLE, NULL));
-    g_assert (priv->embed_single != NULL);
-
-    if (!ephy_embed_single_initialize (priv->embed_single)) {
-      GtkWidget *dialog;
-
-      dialog = gtk_message_dialog_new (NULL,
-                                       GTK_DIALOG_MODAL,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       _("Epiphany can't be used now. "
-                                         "Initialization failed."));
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      
-      exit (0);
-    }
-    
-    priv->single_initialised = TRUE;
-  }
-
-  return G_OBJECT (shell->priv->embed_single);
-}
-
-/**
- * ephy_embed_shell_get_embed_single:
- * @shell: the #EphyEmbedShell
- *
- * Return value: (transfer none):
- **/
-GObject *
-ephy_embed_shell_get_embed_single (EphyEmbedShell *shell)
-{
-  EphyEmbedShellClass *klass = EPHY_EMBED_SHELL_GET_CLASS (shell);
-
-  return klass->get_embed_single (shell);
-}
-
 /**
  * ephy_embed_shell_get_encodings:
  * @shell: the #EphyEmbedShell
@@ -262,6 +297,108 @@ ephy_embed_shell_prepare_close (EphyEmbedShell *shell)
   g_signal_emit (shell, signals[PREPARE_CLOSE], 0);
 }
 
+void
+ephy_embed_shell_restored_window (EphyEmbedShell *shell)
+{
+  g_signal_emit (shell, signals[RESTORED_WINDOW], 0);
+}
+
+static void
+ephy_embed_shell_setup_environment (EphyEmbedShell *shell)
+{
+  EphyEmbedShellMode mode = shell->priv->mode;
+  char *pid_str;
+
+  pid_str = g_strdup_printf ("%u", getpid ());
+  g_setenv ("EPHY_WEB_EXTENSION_ID", pid_str, TRUE);
+  g_setenv ("EPHY_DOT_DIR", ephy_dot_dir (), TRUE);
+  if (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode))
+    g_setenv ("EPHY_PRIVATE_PROFILE", "1", TRUE);
+  g_free (pid_str);
+}
+
+static void
+about_request_cb (WebKitURISchemeRequest *request,
+                  EphyEmbedShell *shell)
+{
+  ephy_about_handler_handle_request (shell->priv->about_handler, request);
+}
+
+static void
+ephy_embed_shell_startup (GApplication* application)
+{
+  EphyEmbedShell *shell = EPHY_EMBED_SHELL (application);
+  EphyEmbedShellMode mode;
+  char *disk_cache_dir;
+  WebKitWebContext *web_context;
+  WebKitCookieManager *cookie_manager;
+  char *filename;
+  char *cookie_policy;
+
+  G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
+
+  /* We're not remoting, setup the Web Context. */
+  mode = shell->priv->mode;
+  web_context = webkit_web_context_get_default ();
+
+  ephy_embed_shell_setup_environment (shell);
+
+  /* Set the web extensions dir ASAP before the process is launched. */
+  webkit_web_context_set_web_extensions_directory (web_context, EPHY_WEB_EXTENSIONS_DIR);
+  ephy_embed_shell_watch_web_extension (shell);
+
+  /* Disk Cache */
+  disk_cache_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode) ?
+                                     ephy_dot_dir () : g_get_user_cache_dir (),
+                                     g_get_prgname (), NULL);
+  webkit_web_context_set_disk_cache_directory (web_context, disk_cache_dir);
+  g_free (disk_cache_dir);
+
+  /* about: URIs handler */
+  shell->priv->about_handler = ephy_about_handler_new ();
+  webkit_web_context_register_uri_scheme (web_context,
+                                          EPHY_ABOUT_SCHEME,
+                                          (WebKitURISchemeRequestCallback)about_request_cb,
+                                          shell, NULL);
+
+  /* Store cookies in moz-compatible SQLite format */
+  cookie_manager = webkit_web_context_get_cookie_manager (web_context);
+  filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
+  webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
+                                                WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
+  g_free (filename);
+
+  cookie_policy = g_settings_get_string (EPHY_SETTINGS_WEB,
+                                         EPHY_PREFS_WEB_COOKIES_POLICY);
+  ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
+  g_free (cookie_policy);
+
+  ephy_embed_prefs_init ();
+}
+
+static void
+ephy_embed_shell_shutdown (GApplication* application)
+{
+  EphyEmbedShellPrivate *priv = EPHY_EMBED_SHELL (application)->priv;
+
+  G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->shutdown (application);
+
+  if (priv->web_extension_watch_name_id > 0) {
+    g_bus_unwatch_name (priv->web_extension_watch_name_id);
+    priv->web_extension_watch_name_id = 0;
+  }
+
+  if (priv->web_extension_form_auth_save_signal_id > 0) {
+    g_dbus_connection_signal_unsubscribe (g_dbus_proxy_get_connection (priv->web_extension),
+                                          priv->web_extension_form_auth_save_signal_id);
+    priv->web_extension_form_auth_save_signal_id = 0;
+  }
+
+  g_clear_object (&priv->web_extension);
+
+  ephy_embed_prefs_shutdown ();
+}
+
 static void
 ephy_embed_shell_set_property (GObject *object,
                                guint prop_id,
@@ -306,24 +443,29 @@ ephy_embed_shell_init (EphyEmbedShell *shell)
   embed_shell = shell;
 
   shell->priv->downloads = NULL;
+
+  /* Initialise nspluginwrapper's plugins if available. */
+  if (g_file_test (NSPLUGINWRAPPER_SETUP, G_FILE_TEST_EXISTS) != FALSE)
+    g_spawn_command_line_sync (NSPLUGINWRAPPER_SETUP, NULL, NULL, NULL, NULL);
 }
 
 static void
 ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = ephy_embed_shell_dispose;
-  object_class->finalize = ephy_embed_shell_finalize;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
 
-  klass->get_embed_single = impl_get_embed_single;
+  application_class->startup = ephy_embed_shell_startup;
+  application_class->shutdown = ephy_embed_shell_shutdown;
 
   object_properties[PROP_MODE] =
     g_param_spec_enum ("mode",
                        "Mode",
-                       "The  global mode f  or this instance of Epiphany .",
+                       "The  global mode for this instance of Epiphany .",
                        EPHY_TYPE_EMBED_SHELL_MODE,
                        EPHY_EMBED_SHELL_MODE_BROWSER,
                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
@@ -384,6 +526,65 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+/**
+ * EphyEmbedShell::finished-restoring-window:
+ * @shell: the #EphyEmbedShell
+ *
+ * The ::finished-restoring-window signal is emitted when the
+ * session finishes restoring a window.
+ **/
+  signals[RESTORED_WINDOW] =
+    g_signal_new ("window-restored",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (EphyEmbedShellClass, restored_window),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
+  /**
+   * EphyEmbedShell::web-view-created:
+   * @shell: the #EphyEmbedShell
+   * @view: the newly created #EphyWebView
+   *
+   * The ::web-view-created signal will be emitted every time a new
+   * #EphyWebView is created.
+   *
+   **/
+  signals[WEB_VIEW_CREATED] =
+    g_signal_new ("web-view-created",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  EPHY_TYPE_WEB_VIEW);
+
+  /*
+   * EphyEmbedShell::form-auth-data-save-requested:
+   * @shell: the #EphyEmbedShell
+   * @request_id: the identifier of the request
+   * @page_id: the identifier of the web page
+   * @hostname: the hostname
+   * @username: the username
+   *
+   * Emitted when a web page requests confirmation to save
+   * the form authentication data for the given @hostname and
+   * @username
+   **/
+  signals[FORM_AUTH_DATA_SAVE_REQUESTED] =
+    g_signal_new ("form-auth-data-save-requested",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE, 4,
+                  G_TYPE_UINT,
+                  G_TYPE_UINT64,
+                  G_TYPE_STRING,
+                  G_TYPE_STRING);
+
   g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
 
@@ -400,25 +601,6 @@ ephy_embed_shell_get_default (void)
   return embed_shell;
 }
 
-/**
- * ephy_embed_shell_get_adblock_manager:
- * @shell: the #EphyEmbedShell
- *
- * Returns the adblock manager.
- *
- * Return value: (transfer none): the adblock manager
- **/
-GObject *
-ephy_embed_shell_get_adblock_manager (EphyEmbedShell *shell)
-{
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
-
-  if (shell->priv->adblock_manager == NULL)
-    shell->priv->adblock_manager = g_object_new (EPHY_TYPE_ADBLOCK_MANAGER, NULL);
-
-  return G_OBJECT (shell->priv->adblock_manager);
-}
-
 void
 ephy_embed_shell_set_page_setup (EphyEmbedShell *shell,
                                  GtkPageSetup *page_setup)
@@ -600,3 +782,67 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
   
   return shell->priv->mode;
 }
+
+/**
+ * ephy_embed_shell_launch_application:
+ * @shell: an #EphyEmbedShell
+ * @file: a #GFile to open
+ * @mime_type: the mime type of @file or %NULL
+ * @user_time: user time to prevent focus stealing
+ * 
+ * Tries to open @file with the right application, making sure we will
+ * not call ourselves in the process. This is needed to avoid
+ * potential infinite loops when opening unknown file types.
+ * 
+ * Returns: %TRUE on success
+ **/
+gboolean
+ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
+                                 GFile *file,
+                                 const char *mime_type,
+                                 guint32 user_time)
+{
+  GAppInfo *app;
+  GList *list = NULL;
+  gboolean ret = FALSE;
+
+  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), FALSE);
+  g_return_val_if_fail (file || mime_type, FALSE);
+
+  app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
+
+  /* Do not allow recursive calls into the browser, they can lead to
+   * infinite loops and they should never happen anyway. */
+
+  /* FIXME: eventually there should be a nice and safe way of getting
+   * the app ID from the GApplication itself, but for now let's
+   * hardcode the .desktop file name and use it here. */
+  if (!app || g_strcmp0 (g_app_info_get_id (app), "epiphany.desktop") == 0)
+    return ret;
+
+  list = g_list_append (list, file);
+  ret = ephy_file_launch_application (app, list, user_time, NULL);
+  g_list_free (list);
+
+  return ret;
+}
+
+GDBusProxy *
+ephy_embed_shell_get_web_extension_proxy (EphyEmbedShell *shell)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+
+  return shell->priv->web_extension;
+}
+
+/**
+ * ephy_embed_shell_clear_cache:
+ * @shell: an #EphyEmbedShell
+ * 
+ * Clears the HTTP cache (temporarily saved web pages).
+ **/
+void
+ephy_embed_shell_clear_cache (EphyEmbedShell *shell)
+{
+  webkit_web_context_clear_cache (webkit_web_context_get_default ());
+}
