@@ -26,7 +26,6 @@
 #include "config.h"
 #include "ephy-embed.h"
 
-#include "ephy-adblock-manager.h"
 #include "ephy-debug.h"
 #include "ephy-download.h"
 #include "ephy-embed-prefs.h"
@@ -37,13 +36,11 @@
 #include "nautilus-floating-bar.h"
 
 #include <glib/gi18n.h>
-#include <webkit/webkit.h>
+#include <webkit2/webkit2.h>
 
-static void     ephy_embed_constructed      (GObject *object);
-static gboolean ephy_embed_inspect_show_cb  (WebKitWebInspector *inspector,
-                                             EphyEmbed *embed);
-static gboolean ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
-                                             EphyEmbed *embed);
+static void     ephy_embed_constructed         (GObject *object);
+static void     ephy_embed_restored_window_cb  (EphyEmbedShell *shell,
+                                                EphyEmbed *embed);
 
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
@@ -58,17 +55,18 @@ typedef struct {
 struct _EphyEmbedPrivate
 {
   GtkBox *top_widgets_vbox;
-  GtkScrolledWindow *scrolled_window;
   GtkPaned *paned;
   WebKitWebView *web_view;
-  GtkWidget *inspector_window;
-  GtkWidget *inspector_web_view;
-  GtkWidget *inspector_scrolled_window;
-  gboolean inspector_attached;
   GSList *destroy_on_transition_list;
   GtkWidget *floating_bar;
   GtkWidget *progress;
+  GtkWidget *fullscreen_message_label;
+  char *fullscreen_string;
 
+  WebKitURIRequest *delayed_request;
+
+  GtkWidget *overview;
+  guint overview_mode : 1;
   GSList *messages;
   GSList *keys;
 
@@ -78,12 +76,20 @@ struct _EphyEmbedPrivate
   guint tab_message_id;
   guint pop_statusbar_later_source_id;
 
+  guint fullscreen_message_id;
+
   guint clear_progress_source_id;
 
   gulong status_handler_id;
   gulong progress_update_handler_id;
 };
 
+enum
+{
+  PROP_0,
+  PROP_OVERVIEW_MODE,
+};
+
 G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
 
 /* Portions of the following code based on GTK+.
@@ -216,29 +222,6 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
   ephy_embed_statusbar_update (embed, msg ? msg->text : NULL);
 }
 
-static void
-resource_request_starting_cb (WebKitWebView *web_view,
-                              WebKitWebFrame *web_frame,
-                              WebKitWebResource *web_resource,
-                              WebKitNetworkRequest *request,
-                              WebKitNetworkResponse *response,
-                              EphyEmbed *embed)
-{
-  EphyAdBlockManager *adblock_manager = EPHY_ADBLOCK_MANAGER (ephy_embed_shell_get_adblock_manager (embed_shell));
-  const char *uri = webkit_network_request_get_uri (request);
-
-  /* FIXME: How do we implement the other CHECK_TYPEs?  Perhaps we
-   * should figure out a way of adding more information about what the
-   * resource is for to WebResource? */
-  if (!ephy_adblock_manager_should_load (adblock_manager, embed, uri,
-                                         AD_URI_CHECK_TYPE_OTHER)) {
-    g_signal_emit_by_name (EPHY_WEB_VIEW (web_view),
-                           "content-blocked", uri);
-
-    webkit_network_request_set_uri (request, "about:blank");
-  }
-}
-
 static void
 ephy_embed_destroy_top_widgets (EphyEmbed *embed)
 {
@@ -259,14 +242,24 @@ remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
 }
 
 static void
-load_status_changed_cb (WebKitWebView *web_view,
-                        GParamSpec *spec,
-                        EphyEmbed *embed)
+load_changed_cb (WebKitWebView *web_view,
+                 WebKitLoadEvent load_event,
+                 EphyEmbed *embed)
 {
-  WebKitLoadStatus status = webkit_web_view_get_load_status (web_view);
+  switch (load_event) {
+  case WEBKIT_LOAD_STARTED: {
+    const char *uri;
 
-  if (status == WEBKIT_LOAD_COMMITTED)
+    uri = webkit_web_view_get_uri (web_view);
+    ephy_embed_set_overview_mode (embed, g_strcmp0 (uri, "ephy-about:overview") == 0);
+    break;
+  }
+  case WEBKIT_LOAD_COMMITTED:
     ephy_embed_destroy_top_widgets (embed);
+    break;
+  default:
+    break;
+  }
 }
 
 static void
@@ -280,29 +273,44 @@ ephy_embed_grab_focus (GtkWidget *widget)
     gtk_widget_grab_focus (child);
 }
 
+
+static gboolean
+fullscreen_message_label_hide (EphyEmbed *embed)
+{
+  if (embed->priv->fullscreen_message_id) {
+    gtk_widget_hide (embed->priv->fullscreen_message_label);
+    g_source_remove (embed->priv->fullscreen_message_id);
+    embed->priv->fullscreen_message_id = 0;
+  }
+
+  return FALSE;
+}
+
+void
+ephy_embed_entering_fullscreen (EphyEmbed *embed)
+{
+  gtk_widget_show (embed->priv->fullscreen_message_label);
+
+  if (embed->priv->fullscreen_message_id)
+    g_source_remove (embed->priv->fullscreen_message_id);
+
+  embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
+                                                              (GSourceFunc)fullscreen_message_label_hide,
+                                                              embed);
+}
+
+void
+ephy_embed_leaving_fullscreen (EphyEmbed *embed)
+{
+  fullscreen_message_label_hide (embed);
+}
+
 static void
 ephy_embed_dispose (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
   EphyEmbedPrivate *priv = embed->priv;
 
-  if (priv->inspector_window) {
-    WebKitWebInspector *inspector;
-
-    inspector = webkit_web_view_get_inspector (priv->web_view);
-
-    g_signal_handlers_disconnect_by_func (inspector,
-                                          ephy_embed_inspect_show_cb,
-                                          priv->inspector_window);
-
-    g_signal_handlers_disconnect_by_func (inspector,
-                                          ephy_embed_inspect_close_cb,
-                                          priv->inspector_window);
-
-    gtk_widget_destroy (GTK_WIDGET (priv->inspector_window));
-    priv->inspector_window = NULL;
-  }
-
   if (priv->pop_statusbar_later_source_id) {
     g_source_remove (priv->pop_statusbar_later_source_id);
     priv->pop_statusbar_later_source_id = 0;
@@ -325,6 +333,13 @@ ephy_embed_dispose (GObject *object)
     priv->progress_update_handler_id = 0;
   }
 
+  if (priv->fullscreen_message_id) {
+    g_source_remove (priv->fullscreen_message_id);
+    priv->fullscreen_message_id = 0;
+  }
+
+  g_clear_object (&priv->delayed_request);
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->dispose (object);
 }
 
@@ -333,8 +348,11 @@ ephy_embed_finalize (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
   EphyEmbedPrivate *priv = embed->priv;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GSList *list;
 
+  g_signal_handlers_disconnect_by_func(shell, ephy_embed_restored_window_cb, embed);
+
   list = priv->destroy_on_transition_list;
   for (; list; list = list->next) {
     GtkWidget *widget = GTK_WIDGET (list->data);
@@ -359,9 +377,49 @@ ephy_embed_finalize (GObject *object)
   g_slist_free (priv->keys);
   priv->keys = NULL;
 
+  g_free (embed->priv->fullscreen_string);
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
 
+static void
+ephy_embed_set_property (GObject *object,
+                         guint prop_id,
+                         const GValue *value,
+                         GParamSpec *pspec)
+{
+  EphyEmbed *embed = EPHY_EMBED (object);
+
+  switch (prop_id)
+  {
+  case PROP_OVERVIEW_MODE:
+    ephy_embed_set_overview_mode (embed, g_value_get_boolean (value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    break;
+  }
+}
+
+static void
+ephy_embed_get_property (GObject *object,
+                         guint prop_id,
+                         GValue *value,
+                         GParamSpec *pspec)
+{
+  EphyEmbed *embed = EPHY_EMBED (object);
+
+  switch (prop_id)
+  {
+  case PROP_OVERVIEW_MODE:
+    g_value_set_boolean (value, ephy_embed_get_overview_mode (embed));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    break;
+  }
+}
+
 static void
 ephy_embed_class_init (EphyEmbedClass *klass)
 {
@@ -371,96 +429,40 @@ ephy_embed_class_init (EphyEmbedClass *klass)
   object_class->constructed = ephy_embed_constructed;
   object_class->finalize = ephy_embed_finalize;
   object_class->dispose = ephy_embed_dispose;
+  object_class->set_property = ephy_embed_set_property;
+  object_class->get_property = ephy_embed_get_property;
   widget_class->grab_focus = ephy_embed_grab_focus;
 
+/**
+ * EphyEmbed:overview-mode:
+ *
+ * If %TRUE activates the overview mode in this #EphyEmbed.
+ **/
+  g_object_class_install_property (object_class,
+                                   PROP_OVERVIEW_MODE,
+                                   g_param_spec_boolean ("overview-mode",
+                                                         "Overview mode",
+                                                         "Whether the embed is showing the overview",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
   g_type_class_add_private (G_OBJECT_CLASS (klass), sizeof(EphyEmbedPrivate));
 }
 
-static WebKitWebView *
-ephy_embed_inspect_web_view_cb (WebKitWebInspector *inspector,
-                                WebKitWebView *web_view,
-                                EphyEmbed *embed)
-{
-  EphyEmbedPrivate *priv = embed->priv;
-
-  priv->inspector_web_view = ephy_web_view_new();
-
-  gtk_container_add (GTK_CONTAINER (priv->inspector_scrolled_window),
-                     priv->inspector_web_view);
-
-  return WEBKIT_WEB_VIEW (priv->inspector_web_view);
-}
-
 static gboolean
 ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
                                 EphyEmbed *embed)
 {
-  GtkAllocation allocation;
-  gtk_widget_get_allocation (GTK_WIDGET (embed->priv->scrolled_window), &allocation);
+  GtkWidget *inspector_view = GTK_WIDGET (webkit_web_inspector_get_web_view (inspector));
+  int inspected_view_height;
+  guint attached_height;
 
-  embed->priv->inspector_attached = TRUE;
+  inspected_view_height = gtk_widget_get_allocated_height (GTK_WIDGET (embed->priv->web_view));
+  attached_height = webkit_web_inspector_get_attached_height (inspector);
+  gtk_paned_set_position (embed->priv->paned, inspected_view_height - attached_height);
 
-  /* Set a sane position for the mover */
-  gtk_paned_set_position (embed->priv->paned, allocation.height * 0.5);
-
-  gtk_widget_hide (embed->priv->inspector_window);
-  gtk_widget_reparent (GTK_WIDGET (embed->priv->inspector_scrolled_window),
-                       GTK_WIDGET (embed->priv->paned));
-
-  return TRUE;
-}
-
-static void
-ephy_embed_detach_inspector (EphyEmbed *embed, WebKitWebInspector *inspector)
-{
-  embed->priv->inspector_attached = FALSE;
-
-  gtk_widget_reparent (GTK_WIDGET (embed->priv->inspector_scrolled_window),
-                       GTK_WIDGET (embed->priv->inspector_window));
-}
-
-static gboolean
-ephy_embed_detach_inspector_cb (WebKitWebInspector *inspector,
-                                EphyEmbed *embed)
-{
-  ephy_embed_detach_inspector (embed, inspector);
-
-  gtk_widget_show_all (embed->priv->inspector_window);
-
-  return TRUE;
-}
-
-static gboolean
-ephy_embed_inspect_show_cb (WebKitWebInspector *inspector,
-                            EphyEmbed *embed)
-{
-  if (!embed->priv->inspector_attached) {
-    gtk_widget_show_all (embed->priv->inspector_window);
-    gtk_window_present (GTK_WINDOW (embed->priv->inspector_window));
-  } else
-    gtk_widget_show (embed->priv->inspector_scrolled_window);
-
-  return TRUE;
-}
-
-static gboolean
-ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
-                             EphyEmbed *embed)
-{
-  EphyEmbedPrivate *priv = embed->priv;
-
-  if (!priv->inspector_window)
-    return TRUE;
-
-  gtk_widget_destroy (priv->inspector_web_view);
-  priv->inspector_web_view = NULL;
-
-  if (!priv->inspector_attached)
-    gtk_widget_hide (priv->inspector_window);
-  else
-    gtk_widget_hide (priv->inspector_scrolled_window);
-
-  ephy_embed_detach_inspector (embed, inspector);
+  gtk_paned_add2 (embed->priv->paned, inspector_view);
+  gtk_widget_show (inspector_view);
 
   return TRUE;
 }
@@ -470,31 +472,41 @@ ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
 {
   EphyDownload *download;
 
-  download = ephy_download_new_for_uri (url);
+  download = ephy_download_new_for_uri (url, NULL);
   ephy_download_set_auto_destination (download);
   ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_OPEN);
 }
 
+static void
+ephy_embed_set_fullscreen_message (EphyEmbed *embed,
+                                   gboolean is_html5_fullscreen)
+{
+  char *message;
+
+  if (G_UNLIKELY (embed->priv->fullscreen_string == NULL))
+    embed->priv->fullscreen_string = g_strdup (_("Press %s to exit fullscreen"));
+
+  /* Translators: 'ESC' and 'F11' are keyboard keys. */
+  message = g_strdup_printf (embed->priv->fullscreen_string, is_html5_fullscreen ? _("ESC") : _("F11"));
+  gtk_label_set_text (GTK_LABEL (embed->priv->fullscreen_message_label),
+                      message);
+  g_free (message);
+}
+
 static gboolean
-download_requested_cb (WebKitWebView *web_view,
-                       WebKitDownload *download,
+entering_fullscreen_cb (WebKitWebView *web_view,
+                        EphyEmbed *embed)
+{
+  ephy_embed_set_fullscreen_message (embed, TRUE);
+  return FALSE;
+}
+
+static gboolean
+leaving_fullscreen_cb (WebKitWebView *web_view,
                        EphyEmbed *embed)
 {
-  EphyDownload *ed;
-  GtkWidget *window;
-
-  /* Is download locked down? */
-  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK))
-    return FALSE;
-
-  window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
-
-  ed = ephy_download_new_for_download (download);
-  ephy_download_set_window (ed, window);
-  ephy_download_set_auto_destination (ed);
-
-  return TRUE;
+  ephy_embed_set_fullscreen_message (embed, FALSE);
+  return FALSE;
 }
 
 static gboolean
@@ -529,17 +541,18 @@ status_message_notify_cb (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed
   } else {
     /* A short timeout before hiding the statusbar ensures that while moving
       over a series of links, the overlay widget doesn't flicker on and off. */
-    priv->pop_statusbar_later_source_id = g_timeout_add (250, pop_statusbar_later_cb, embed);
+    if (priv->pop_statusbar_later_source_id == 0) {
+      priv->pop_statusbar_later_source_id = g_timeout_add (250, pop_statusbar_later_cb, embed);
+    }
   }
 }
 
 static void
-window_resize_requested (WebKitWebWindowFeatures *features, GParamSpec *pspec, EphyEmbed *embed)
+window_geometry_changed (WebKitWindowProperties *properties, GParamSpec *pspec, EphyEmbed *embed)
 {
   GtkWidget *window;
   gboolean is_popup;
-  const char *property_name;
-  int width, height;
+  GdkRectangle geometry;
 
   window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
   if (!window || !gtk_widget_is_toplevel (window))
@@ -549,17 +562,11 @@ window_resize_requested (WebKitWebWindowFeatures *features, GParamSpec *pspec, E
   if (!is_popup)
     return;
 
-  property_name = g_param_spec_get_name (pspec);
-
-  if (g_str_equal (property_name, "x") || g_str_equal (property_name, "y")) {
-    int x, y;
-    g_object_get (features, "x", &x, "y", &y, NULL);
-    gtk_window_move (GTK_WINDOW (window), x, y);
-    return;
-  }
-
-  g_object_get (features, "width", &width, "height", &height, NULL);
-  gtk_window_resize (GTK_WINDOW (window), width, height);
+  webkit_window_properties_get_geometry (properties, &geometry);
+  if (geometry.x >= 0 && geometry.y >= 0)
+    gtk_window_move (GTK_WINDOW (window), geometry.x, geometry.y);
+  if (geometry.width > 0 && geometry.height > 0)
+    gtk_window_resize (GTK_WINDOW (window), geometry.width, geometry.height);
 }
 
 static gboolean
@@ -586,10 +593,11 @@ progress_update (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
   }
 
   uri = webkit_web_view_get_uri (priv->web_view);
-  if (!uri || g_str_equal (uri, "about:blank"))
+  if (!uri || g_str_has_prefix (uri, "ephy-about:") ||
+      g_str_has_prefix (uri, "about:"))
     return;
 
-  progress = webkit_web_view_get_progress (priv->web_view);
+  progress = webkit_web_view_get_estimated_load_progress (priv->web_view);
   loading = ephy_web_view_is_loading (EPHY_WEB_VIEW (priv->web_view));
 
   if (progress == 1.0 || !loading)
@@ -603,27 +611,89 @@ progress_update (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
                                  (loading || progress == 1.0) ? progress : 0.0);
 }
 
+static void
+ephy_embed_maybe_load_delayed_request (EphyEmbed *embed)
+{
+  EphyEmbedPrivate *priv = embed->priv;
+  EphyWebView *web_view;
+
+  if (!priv->delayed_request)
+    return;
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  ephy_web_view_load_request (web_view, priv->delayed_request);
+  g_clear_object (&priv->delayed_request);
+
+  /* This is to allow UI elements watching load status to show that the page is
+   * loading as soon as possible.
+   */
+  g_signal_emit_by_name (web_view, "load-changed", WEBKIT_LOAD_STARTED);
+}
+
+static void
+ephy_embed_restored_window_cb (EphyEmbedShell *shell, EphyEmbed *embed)
+{
+  if (!gtk_widget_get_mapped (GTK_WIDGET (embed)))
+    return;
+
+  ephy_embed_maybe_load_delayed_request (embed);
+}
+
+static void
+ephy_embed_mapped_cb (GtkWidget *widget, gpointer data)
+{
+  ephy_embed_maybe_load_delayed_request ((EphyEmbed*)widget);
+}
+
 static void
 ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed*)object;
   EphyEmbedPrivate *priv = embed->priv;
-  GtkWidget *scrolled_window;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GtkWidget *paned;
   WebKitWebView *web_view;
-  WebKitWebWindowFeatures *window_features;
+  WebKitWindowProperties *window_properties;
   WebKitWebInspector *inspector;
   GtkWidget *overlay;
 
+  g_signal_connect (shell, "window-restored",
+                    G_CALLBACK (ephy_embed_restored_window_cb), embed);
+
+  g_signal_connect (embed, "map",
+                    G_CALLBACK (ephy_embed_mapped_cb), NULL);
+
   /* Skeleton */
   web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
-  scrolled_window = GTK_WIDGET (priv->scrolled_window);
   overlay = gtk_overlay_new ();
+
   gtk_widget_add_events (overlay, 
                          GDK_ENTER_NOTIFY_MASK |
                          GDK_LEAVE_NOTIFY_MASK);
-  gtk_widget_set_name (overlay, "ephy-overlay");
-  gtk_container_add (GTK_CONTAINER (overlay), scrolled_window);
+  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (web_view));
+
+  /* The overview. In incognito mode we don't use it. */
+  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) !=
+      EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+    priv->overview = ephy_overview_new ();
+    gtk_widget_set_halign (priv->overview, GTK_ALIGN_FILL);
+    gtk_widget_set_valign (priv->overview, GTK_ALIGN_FILL);
+    gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->overview);
+
+    g_object_bind_property (embed, "overview-mode",
+                            priv->overview, "visible",
+                            G_BINDING_SYNC_CREATE);
+  }
+
+  /* Floating message popup for fullscreen mode. */
+  priv->fullscreen_message_label = gtk_label_new (NULL);
+  gtk_widget_set_name (priv->fullscreen_message_label, "fullscreen-popup");
+  gtk_widget_set_halign (priv->fullscreen_message_label, GTK_ALIGN_CENTER);
+  gtk_widget_set_valign (priv->fullscreen_message_label, GTK_ALIGN_CENTER);
+  gtk_widget_set_no_show_all (priv->fullscreen_message_label, TRUE);
+  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->fullscreen_message_label);
+  ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
   priv->floating_bar = nautilus_floating_bar_new (NULL, FALSE);
@@ -634,7 +704,8 @@ ephy_embed_constructed (GObject *object)
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->floating_bar);
 
   priv->progress = gtk_progress_bar_new ();
-  gtk_widget_set_name (priv->progress, "ephy-progress-bar");
+  gtk_style_context_add_class (gtk_widget_get_style_context (priv->progress),
+                               GTK_STYLE_CLASS_OSD);
   gtk_widget_set_halign (priv->progress, GTK_ALIGN_FILL);
   gtk_widget_set_valign (priv->progress, GTK_ALIGN_START);
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->progress);
@@ -642,11 +713,8 @@ ephy_embed_constructed (GObject *object)
   paned = GTK_WIDGET (priv->paned);
 
   priv->web_view = web_view;
-  priv->progress_update_handler_id =  g_signal_connect (web_view, "notify::progress",
-                                                        G_CALLBACK (progress_update), object);
-
-  gtk_container_add (GTK_CONTAINER (scrolled_window),
-                     GTK_WIDGET (web_view));
+  priv->progress_update_handler_id = g_signal_connect (web_view, "notify::estimated-load-progress",
+                                                       G_CALLBACK (progress_update), object);
   gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (overlay),
                    TRUE, FALSE);
 
@@ -660,76 +728,43 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_show_all (paned);
 
   g_object_connect (web_view,
-                    "signal::notify::load-status", G_CALLBACK (load_status_changed_cb), embed,
-                    "signal::resource-request-starting", G_CALLBACK (resource_request_starting_cb), embed,
-                    "signal::download-requested", G_CALLBACK (download_requested_cb), embed,
+                    "signal::load-changed", G_CALLBACK (load_changed_cb), embed,
+                    "signal::enter-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
+                    "signal::leave-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
                     NULL);
 
   priv->status_handler_id = g_signal_connect (web_view, "notify::status-message",
                                               G_CALLBACK (status_message_notify_cb),
                                               embed);
-
-  /* Window features */
-  window_features = webkit_web_view_get_window_features (web_view);
-  g_object_connect (window_features,
-                    "signal::notify::x", G_CALLBACK (window_resize_requested), embed,
-                    "signal::notify::y", G_CALLBACK (window_resize_requested), embed,
-                    "signal::notify::width", G_CALLBACK (window_resize_requested), embed,
-                    "signal::notify::height", G_CALLBACK (window_resize_requested), embed,
-                    NULL);
+  /* Window properties */
+  window_properties = webkit_web_view_get_window_properties (web_view);
+  g_signal_connect (window_properties, "notify::geometry",
+                    G_CALLBACK (window_geometry_changed),
+                    embed);
 
   /* The inspector */
-  priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   inspector = webkit_web_view_get_inspector (web_view);
 
-  priv->inspector_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->inspector_scrolled_window),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (priv->inspector_window),
-                     priv->inspector_scrolled_window);
-
-  gtk_window_set_title (GTK_WINDOW (priv->inspector_window),
-                        _("Web Inspector"));
-  gtk_window_set_default_size (GTK_WINDOW (priv->inspector_window),
-                               800, 600);
-
-  g_signal_connect (priv->inspector_window,
-                    "delete-event", G_CALLBACK (gtk_widget_hide_on_delete),
-                    NULL);
-
-  g_object_connect (inspector,
-                    "signal::inspect-web-view", G_CALLBACK (ephy_embed_inspect_web_view_cb),
-                    embed,
-                    "signal::show-window", G_CALLBACK (ephy_embed_inspect_show_cb),
-                    embed,
-                    "signal::close-window", G_CALLBACK (ephy_embed_inspect_close_cb),
-                    embed,
-                    "signal::attach-window", G_CALLBACK (ephy_embed_attach_inspector_cb),
-                    embed,
-                    "signal::detach-window", G_CALLBACK (ephy_embed_detach_inspector_cb),
-                    embed,
-                    NULL);
-
-  ephy_embed_prefs_add_embed (embed);
+  g_signal_connect (inspector, "attach",
+                    G_CALLBACK (ephy_embed_attach_inspector_cb),
+                    embed);
 }
 
 static void
 ephy_embed_init (EphyEmbed *embed)
 {
-  embed->priv = EPHY_EMBED_GET_PRIVATE (embed);
+  EphyEmbedPrivate *priv;
+
+  priv = embed->priv = EPHY_EMBED_GET_PRIVATE (embed);
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (embed),
                                   GTK_ORIENTATION_VERTICAL);
 
-  embed->priv->scrolled_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
-  embed->priv->paned = GTK_PANED (gtk_paned_new (GTK_ORIENTATION_VERTICAL));
-  embed->priv->top_widgets_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
-  embed->priv->seq_context_id = 1;
-  embed->priv->seq_message_id = 1;
-  embed->priv->tab_message_id = ephy_embed_statusbar_get_context_id (embed, EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION);
-
-  gtk_scrolled_window_set_policy (embed->priv->scrolled_window,
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  priv->paned = GTK_PANED (gtk_paned_new (GTK_ORIENTATION_VERTICAL));
+  priv->top_widgets_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
+  priv->seq_context_id = 1;
+  priv->seq_message_id = 1;
+  priv->tab_message_id = ephy_embed_statusbar_get_context_id (embed, EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION);
 }
 
 /**
@@ -771,7 +806,7 @@ ephy_embed_add_top_widget (EphyEmbed *embed, GtkWidget *widget, gboolean destroy
   }
 
   gtk_box_pack_end (embed->priv->top_widgets_vbox,
-                    GTK_WIDGET (widget), TRUE, TRUE, 0);
+                    GTK_WIDGET (widget), FALSE, FALSE, 0);
 }
 
 /**
@@ -780,7 +815,7 @@ ephy_embed_add_top_widget (EphyEmbed *embed, GtkWidget *widget, gboolean destroy
  * @widget: a #GtkWidget
  *
  * Removes an #GtkWidget from the top of the embed. The #GtkWidget
- * must be have been added using ephy_embed_add_widget(), and not
+ * must have been added using ephy_embed_add_top_widget(), and not
  * have been removed by other means. See gtk_container_remove() for
  * details.
  */
@@ -799,3 +834,80 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
   gtk_container_remove (GTK_CONTAINER (embed->priv->top_widgets_vbox),
                         GTK_WIDGET (widget));
 }
+
+void
+ephy_embed_set_overview_mode (EphyEmbed *embed, gboolean overview_mode)
+{
+  EphyEmbedPrivate *priv;
+
+  g_return_if_fail (EPHY_IS_EMBED (embed));
+
+  priv = embed->priv;
+
+  if (priv->overview_mode == overview_mode)
+    return;
+
+  priv->overview_mode = overview_mode;
+
+  g_object_notify (G_OBJECT (embed), "overview-mode");
+}
+
+gboolean
+ephy_embed_get_overview_mode (EphyEmbed *embed)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+
+  return embed->priv->overview_mode;
+}
+
+/**
+ * ephy_embed_set_delayed_load_request:
+ * @embed: a #EphyEmbed
+ * @request: a #WebKitNetworkRequest
+ *
+ * Sets the #WebKitNetworkRequest that should be loaded when the tab this embed
+ * is on is switched to.
+ */
+void
+ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitURIRequest *request)
+{
+  g_return_if_fail (EPHY_IS_EMBED (embed));
+  g_return_if_fail (WEBKIT_IS_URI_REQUEST (request));
+
+  g_clear_object (&embed->priv->delayed_request);
+
+  g_object_ref (request);
+  embed->priv->delayed_request = request;
+}
+
+/**
+ * ephy_embed_has_load_pending:
+ * @embed: a #EphyEmbed
+ *
+ * Checks whether a load has been delayed for this #EphyEmbed.
+ *
+ * Returns: %TRUE or %FALSE
+ */
+gboolean
+ephy_embed_has_load_pending (EphyEmbed *embed)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+
+  return !!embed->priv->delayed_request;
+}
+
+/**
+ * ephy_embed_get_overview:
+ * @embed: a #EphyEmbed
+ *
+ * Gets the #EphyOverview in this @embed
+ *
+ * Returns: (transfer none): the overview widget
+ **/
+EphyOverview *
+ephy_embed_get_overview (EphyEmbed *embed)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+
+  return EPHY_OVERVIEW (embed->priv->overview);
+}
