@@ -27,8 +27,7 @@
 #ifndef EPHY_WEB_VIEW_H
 #define EPHY_WEB_VIEW_H
 
-#include <gtk/gtk.h>
-#include <webkit/webkit.h>
+#include <webkit2/webkit2.h>
 
 G_BEGIN_DECLS
 
@@ -51,16 +50,11 @@ typedef enum
 
 typedef enum
 {
-  EPHY_WEB_VIEW_CHROME_MENUBAR    = 1 << 0,
-  EPHY_WEB_VIEW_CHROME_TOOLBAR    = 1 << 1,
-  EPHY_WEB_VIEW_CHROME_STATUSBAR    = 1 << 2,
-  EPHY_WEB_VIEW_CHROME_BOOKMARKSBAR = 1 << 3
+  EPHY_WEB_VIEW_CHROME_TOOLBAR    = 1 << 0,
+  EPHY_WEB_VIEW_CHROME_DOWNLOADS_BOX = 1 << 1
 } EphyWebViewChrome;
 
-#define EPHY_WEB_VIEW_CHROME_ALL (EPHY_WEB_VIEW_CHROME_MENUBAR |  \
-                                  EPHY_WEB_VIEW_CHROME_TOOLBAR |  \
-                                  EPHY_WEB_VIEW_CHROME_STATUSBAR |  \
-                                  EPHY_WEB_VIEW_CHROME_BOOKMARKSBAR)
+#define EPHY_WEB_VIEW_CHROME_ALL (EPHY_WEB_VIEW_CHROME_TOOLBAR)
 
 typedef enum
 {
@@ -82,7 +76,8 @@ typedef enum
 
 typedef enum {
   EPHY_WEB_VIEW_ERROR_PAGE_NETWORK_ERROR,
-  EPHY_WEB_VIEW_ERROR_PAGE_CRASH
+  EPHY_WEB_VIEW_ERROR_PAGE_CRASH,
+  EPHY_WEB_VIEW_ERROR_PROCESS_CRASH
 } EphyWebViewErrorPage;
 
 struct _EphyWebView
@@ -128,68 +123,51 @@ GType                      ephy_web_view_chrome_get_type          (void);
 GType                      ephy_web_view_security_level_get_type  (void);
 GtkWidget *                ephy_web_view_new                      (void);
 void                       ephy_web_view_load_request             (EphyWebView               *view,
-                                                                   WebKitNetworkRequest      *request);
+                                                                   WebKitURIRequest          *request);
 void                       ephy_web_view_load_url                 (EphyWebView               *view,
                                                                    const char                *url);
 void                       ephy_web_view_copy_back_history        (EphyWebView               *source,
                                                                    EphyWebView               *dest);
 gboolean                   ephy_web_view_is_loading               (EphyWebView               *view);
+gboolean                   ephy_web_view_load_failed              (EphyWebView               *view);
 const char *               ephy_web_view_get_loading_title        (EphyWebView               *view);
 GdkPixbuf *                ephy_web_view_get_icon                 (EphyWebView               *view);
 EphyWebViewDocumentType    ephy_web_view_get_document_type        (EphyWebView               *view);
 EphyWebViewNavigationFlags ephy_web_view_get_navigation_flags     (EphyWebView               *view);
 const char *               ephy_web_view_get_status_message       (EphyWebView               *view);
 const char *               ephy_web_view_get_link_message         (EphyWebView               *view);
-gboolean                   ephy_web_view_get_visibility           (EphyWebView               *view);
 void                       ephy_web_view_set_link_message         (EphyWebView               *view,
-                                                                   char                      *link_message);
+                                                                   const char                *link_message);
 void                       ephy_web_view_set_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel   level);
-void                       ephy_web_view_set_visibility           (EphyWebView               *view,
-                                                                   gboolean                   visibility);
 const char *               ephy_web_view_get_typed_address        (EphyWebView               *view);
 void                       ephy_web_view_set_typed_address        (EphyWebView               *view,
                                                                    const char                *address);
 gboolean                   ephy_web_view_get_is_blank             (EphyWebView               *view);
-gboolean                   ephy_web_view_has_modified_forms       (EphyWebView               *view);
-char *                     ephy_web_view_get_location             (EphyWebView               *view,
-                                                                   gboolean                   toplevel);
+void                       ephy_web_view_has_modified_forms       (EphyWebView               *view,
+                                                                   GCancellable              *cancellable,
+                                                                   GAsyncReadyCallback        callback,
+                                                                   gpointer                   user_data);
+gboolean                  ephy_web_view_has_modified_forms_finish (EphyWebView               *view,
+                                                                   GAsyncResult              *result,
+                                                                   GError                   **error);
 void                       ephy_web_view_get_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel  *level,
-                                                                   char                     **description);
+                                                                   GTlsCertificate          **certificate,
+                                                                   GTlsCertificateFlags      *errors);
 void                       ephy_web_view_print                    (EphyWebView               *view);
-void                       ephy_web_view_set_title                (EphyWebView               *view,
-                                                                   const char                *view_title);
 const char *               ephy_web_view_get_title                (EphyWebView               *view);
 const char *               ephy_web_view_get_address              (EphyWebView               *view);
 const char *               ephy_web_view_get_title_composite      (EphyWebView               *view);
 
+void                       ephy_web_view_set_placeholder          (EphyWebView               *view,
+                                                                   const char                *uri,
+                                                                   const char                *title);
+
 void                       ephy_web_view_load_error_page          (EphyWebView               *view,
                                                                    const char                *uri,
                                                                    EphyWebViewErrorPage       page,
                                                                    GError                    *error);
-/* These should be private */
-void                       ephy_web_view_set_address              (EphyWebView               *view,
-                                                                   const char                *address);
-void                       ephy_web_view_location_changed         (EphyWebView               *view,
-                                                                   const char                *location);
-void                       ephy_web_view_set_loading_title        (EphyWebView               *view,
-                                                                   const char                *title,
-                                                                   gboolean                   is_address);
-void                       ephy_web_view_popups_manager_reset     (EphyWebView               *view);
-void                       ephy_web_view_save                     (EphyWebView               *view,
-                                                                   const char                *uri);
-void                       ephy_web_view_load_homepage            (EphyWebView               *view);
-
-char *                     ephy_web_view_create_web_application   (EphyWebView               *view,
-                                                                   const char                *title,
-                                                                   GdkPixbuf                 *icon);
-GdkPixbuf *                ephy_web_view_get_snapshot             (EphyWebView               *view,
-                                                                   int                        x,
-                                                                   int                        y,
-                                                                   int                        width,
-                                                                   int                        height);
-gboolean                   ephy_web_view_is_loading_homepage      (EphyWebView               *view);
 
 G_END_DECLS
 
