@@ -34,6 +34,7 @@
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
+#include "ephy-form-auth-data.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
@@ -46,14 +47,15 @@
 #include <fcntl.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-#include <gnome-keyring.h>
-#include <libsoup/soup-gnome.h>
+#include <libsecret/secret.h>
+#include <libsoup/soup.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
 static int do_step_n = -1;
 static int version = -1;
 static char *profile_dir = NULL;
+static GMainLoop *loop = NULL;
 
 /*
  * What to do to add new migration steps:
@@ -64,13 +66,13 @@ static char *profile_dir = NULL;
 typedef void (*EphyProfileMigrator) (void);
 
 static gboolean
-profile_dir_exists ()
+profile_dir_exists (void)
 {
   return g_file_test (ephy_dot_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
 }
 
 static void
-migrate_cookies ()
+migrate_cookies (void)
 {
   const char *cookies_file_sqlite = "cookies.sqlite";
   const char *cookies_file_txt = "cookies.txt";
@@ -105,7 +107,7 @@ migrate_cookies ()
     SoupCookieJar *txt, *sqlite;
 
     txt = soup_cookie_jar_text_new (src_txt, TRUE);
-    sqlite = soup_cookie_jar_sqlite_new (dest, FALSE);
+    sqlite = soup_cookie_jar_db_new (dest, FALSE);
     cookies = soup_cookie_jar_all_cookies (txt);
 
     for (p = cookies; p; p = p->next) {
@@ -252,7 +254,6 @@ parse_and_decrypt_signons (const char *signons,
       char *password = NULL;
       char *form_username = NULL;
       char *form_password = NULL;
-      guint32 item_id;
 
       /* The username */
       if (handle_forms) {
@@ -297,26 +298,27 @@ parse_and_decrypt_signons (const char *signons,
           !g_str_equal (form_password, "*")) {
         char *u = soup_uri_to_string (uri, FALSE);
         /* We skip the '*' at the beginning of form_password. */
-        _ephy_profile_utils_store_form_auth_data (u,
-                                                 form_username,
-                                                 form_password+1,
-                                                 username,
-                                                 password);
+        ephy_form_auth_data_store (u,
+                                   form_username,
+                                   form_password+1,
+                                   username,
+                                   password,
+                                   NULL, NULL);
         g_free (u);
       } else if (!handle_forms && realm &&
                  username && password &&
                  !g_str_equal (username, "") &&
                  form_username == NULL && form_password == NULL) {
-        gnome_keyring_set_network_password_sync (NULL,
-                                                 username,
-                                                 realm,
-                                                 uri->host,
-                                                 NULL,
-                                                 uri->scheme,
-                                                 NULL,
-                                                 uri->port,
-                                                 password,
-                                                 &item_id);
+        char *u = soup_uri_to_string (uri, FALSE);
+        secret_password_store_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+                                    SECRET_COLLECTION_DEFAULT,
+                                    u, password, NULL, NULL,
+                                    "user", username,
+                                    "domain", realm,
+                                    "server", uri->host,
+                                    "protocol", uri->scheme,
+                                    "port", (gint)uri->port,
+                                    NULL);
       }
 
       g_free (username);
@@ -335,7 +337,7 @@ parse_and_decrypt_signons (const char *signons,
 #endif
 
 static void
-migrate_passwords ()
+migrate_passwords (void)
 {
 #ifdef ENABLE_NSS
   char *dest, *contents, *gecko_passwords_backup;
@@ -383,7 +385,7 @@ migrate_passwords ()
 }
 
 static void
-migrate_passwords2 ()
+migrate_passwords2 (void)
 {
 #ifdef ENABLE_NSS
   char *dest, *contents;
@@ -549,7 +551,7 @@ static GMarkupParser history_parse_funcs =
 };
 
 static void
-migrate_history ()
+migrate_history (void)
 {
   GFileInputStream *input;
   GMarkupParseContext *context;
@@ -559,11 +561,11 @@ migrate_history ()
   char buffer[1024];
   HistoryParseData parse_data;
 
-  gchar *temporary_file = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+  gchar *temporary_file = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
   /* Do nothing if the history file already exists. Safer than wiping
    * it out. */
   if (g_file_test (temporary_file, G_FILE_TEST_EXISTS)) {
-    g_warning ("Did not migrate Epiphany's history, the ephy-history.db file already exists");
+    g_warning ("Did not migrate Epiphany's history, the %s file already exists", EPHY_HISTORY_FILE);
     g_free (temporary_file);
     return;
   }
@@ -620,7 +622,7 @@ migrate_history ()
 }
 
 static void
-migrate_tabs_visibility ()
+migrate_tabs_visibility (void)
 {
   gboolean always_show_tabs;
 
@@ -670,7 +672,7 @@ migrate_profile (const char *old_dir,
 }
 
 static void
-migrate_profile_gnome2_to_xdg ()
+migrate_profile_gnome2_to_xdg (void)
 {
   char *old_dir;
   char *new_dir;
@@ -723,7 +725,7 @@ fix_desktop_file_and_return_new_location (const char *dir)
 }
 
 static void
-migrate_web_app_links ()
+migrate_web_app_links (void)
 {
   GList *apps, *p;
 
@@ -773,13 +775,13 @@ migrate_web_app_links ()
 }
 
 static void
-migrate_new_urls_table ()
+migrate_new_urls_table (void)
 {
   EphySQLiteConnection *history_database;
   char *filename;
   GError *error = NULL;
 
-  filename = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+  filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
   history_database = ephy_sqlite_connection_new ();
   ephy_sqlite_connection_open (history_database, filename, &error);
 
@@ -815,6 +817,143 @@ migrate_new_urls_table ()
   g_free (filename);
 }
 
+/* Migrating form password data. */
+
+static int form_passwords_migrating = 0;
+
+
+static void
+password_cleared_cb (SecretService *service,
+                     GAsyncResult *res,
+                     gpointer userdata)
+{
+  secret_service_clear_finish (service, res, NULL);
+
+  if (g_atomic_int_dec_and_test (&form_passwords_migrating))
+    g_main_loop_quit (loop);
+}
+
+static void
+store_form_auth_data_cb (GObject *object,
+                         GAsyncResult *res,
+                         GHashTable *attributes)
+{
+  GError *error = NULL;
+
+  if (ephy_form_auth_data_store_finish (res, &error) == FALSE) {
+    g_warning ("Couldn't store a form password: %s", error->message);
+    g_error_free (error);
+    goto out;
+  }
+
+  g_atomic_int_inc (&form_passwords_migrating);
+  secret_service_clear (NULL, NULL,
+                        attributes, NULL, (GAsyncReadyCallback)password_cleared_cb,
+                        NULL);
+
+out:
+  if (g_atomic_int_dec_and_test (&form_passwords_migrating))
+    g_main_loop_quit (loop);
+
+  g_hash_table_unref (attributes);
+}
+
+static void
+load_collection_items_cb (SecretCollection *collection,
+                          GAsyncResult *res,
+                          gpointer data)
+{
+  SecretItem *item;
+  SecretValue *secret;
+  GList *l;
+  GHashTable *attributes, *t;
+  const char *server, *username, *form_username, *form_password, *password;
+  char *actual_server;
+  SoupURI *uri;
+  GError *error = NULL;
+  GList *items;
+
+  secret_collection_load_items_finish (collection, res, &error);
+
+  if (error) {
+    g_warning ("Couldn't retrieve form data: %s", error->message);
+    g_error_free (error);
+    return;
+  }
+  items = secret_collection_get_items (collection);
+
+  for (l = items; l; l = l->next) {
+    item = (SecretItem*)l->data;
+
+    attributes = secret_item_get_attributes (item);
+    server = g_hash_table_lookup (attributes, "server");
+    if (server &&
+        g_strstr_len (server, -1, "form%5Fusername") &&
+        g_strstr_len (server, -1, "form%5Fpassword")) {
+      g_atomic_int_inc (&form_passwords_migrating);
+      /* This is one of the hackish ones that need to be migrated.
+         Fetch the rest of the data and take care of it. */
+      username = g_hash_table_lookup (attributes, "user");
+      uri = soup_uri_new (server);
+      t = soup_form_decode (uri->query);
+      form_username = g_hash_table_lookup (t, FORM_USERNAME_KEY);
+      form_password = g_hash_table_lookup (t, FORM_PASSWORD_KEY);
+      soup_uri_set_query (uri, NULL);
+      actual_server = soup_uri_to_string (uri, FALSE);
+      secret_item_load_secret_sync (item, NULL, NULL);
+      secret = secret_item_get_secret (item);
+      password = secret_value_get (secret, NULL);
+      ephy_form_auth_data_store (actual_server,
+                                 form_username,
+                                 form_password,
+                                 username,
+                                 password,
+                                 (GAsyncReadyCallback)store_form_auth_data_cb,
+                                 g_hash_table_ref (attributes));
+      g_free (actual_server);
+      secret_value_unref (secret);
+      g_hash_table_unref (t);
+      soup_uri_free (uri);
+    }
+    g_hash_table_unref (attributes);
+  }
+
+  /* And decrease here so that we finish eventually. */
+  if (g_atomic_int_dec_and_test (&form_passwords_migrating))
+    g_main_loop_quit (loop);
+
+  g_list_free_full (items, (GDestroyNotify)g_object_unref);
+}
+
+static void
+migrate_form_passwords_to_libsecret (void)
+{
+  SecretService *service;
+  GList *collections, *c;
+  GError *error = NULL;
+
+  service = secret_service_get_sync (SECRET_SERVICE_OPEN_SESSION | SECRET_SERVICE_LOAD_COLLECTIONS, NULL, &error);
+  if (error) {
+    g_warning ("Could not get the secret service: %s", error->message);
+    g_error_free (error);
+    return;
+  }
+
+  collections = secret_service_get_collections (service);
+
+  for (c = collections; c; c = c->next) {
+    g_atomic_int_inc (&form_passwords_migrating);
+    secret_collection_load_items ((SecretCollection*)c->data, NULL, (GAsyncReadyCallback)load_collection_items_cb,
+                                  NULL);
+  }
+
+  loop = g_main_loop_new (NULL, FALSE);
+  g_main_loop_run (loop);
+
+  g_list_free_full (collections, (GDestroyNotify)g_object_unref);
+  g_object_unref (service);
+}
+
 const EphyProfileMigrator migrators[] = {
   migrate_cookies,
   migrate_passwords,
@@ -828,10 +967,11 @@ const EphyProfileMigrator migrators[] = {
   migrate_tabs_visibility,
   migrate_web_app_links,
   migrate_new_urls_table,
+  migrate_form_passwords_to_libsecret,
 };
 
 static gboolean
-ephy_migrator ()
+ephy_migrator (void)
 {
   int latest, i;
   EphyProfileMigrator m;
@@ -903,8 +1043,6 @@ main (int argc, char *argv[])
   GError *error = NULL;
   EphyFileHelpersFlags file_helpers_flags = EPHY_FILE_HELPERS_NONE;
 
-  g_type_init ();
-
   option_group = g_option_group_new ("ephy-profile-migrator",
                                      N_("Epiphany profile migrator"),
                                      N_("Epiphany profile migrator options"),
