@@ -40,7 +40,9 @@ ephy_history_service_initialize_urls_table (EphyHistoryService *self)
     "title LONGVARCAR,"
     "visit_count INTEGER DEFAULT 0 NOT NULL,"
     "typed_count INTEGER DEFAULT 0 NOT NULL,"
-    "last_visit_time INTEGER)", &error);
+    "last_visit_time INTEGER,"
+    "thumbnail_update_time INTEGER DEFAULT 0,"
+    "hidden_from_overview INTEGER DEFAULT 0)", &error);
 
   if (error) {
     g_error("Could not create urls table: %s", error->message);
@@ -68,11 +70,11 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
 
   if (url != NULL && url->id != -1) {
     statement = ephy_sqlite_connection_create_statement (priv->history_database,
-      "SELECT id, url, title, visit_count, typed_count, last_visit_time FROM urls "
+      "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
       "WHERE id=?", &error);
   } else {
     statement = ephy_sqlite_connection_create_statement (priv->history_database,
-      "SELECT id, url, title, visit_count, typed_count, last_visit_time FROM urls "
+      "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
       "WHERE url=?", &error);
   }
 
@@ -115,6 +117,8 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
   url->visit_count = ephy_sqlite_statement_get_column_as_int (statement, 3),
   url->typed_count = ephy_sqlite_statement_get_column_as_int (statement, 4),
   url->last_visit_time = ephy_sqlite_statement_get_column_as_int (statement, 5);
+  url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
+  url->thumbnail_time = ephy_sqlite_statement_get_column_as_int (statement, 7);
 
   g_object_unref (statement);
   return url;
@@ -147,6 +151,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
       ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE) {
     g_error ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
+    g_object_unref (statement);
     return;
   }
 
@@ -172,7 +177,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
   g_assert (priv->history_database != NULL);
 
   statement = ephy_sqlite_connection_create_statement (priv->history_database,
-    "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=? "
+    "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
     "WHERE id=?", &error);
   if (error) {
     g_error ("Could not build urls table modification statement: %s", error->message);
@@ -184,9 +189,12 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
       ephy_sqlite_statement_bind_int (statement, 1, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, url->typed_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 3, url->last_visit_time, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 4, url->id, &error) == FALSE) {
+      ephy_sqlite_statement_bind_int (statement, 4, url->hidden, &error) == FALSE ||
+      ephy_sqlite_statement_bind_int (statement, 5, url->thumbnail_time, &error) == FALSE ||
+      ephy_sqlite_statement_bind_int (statement, 6, url->id, &error) == FALSE) {
     g_error ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
+    g_object_unref (statement);
     return;
   }
 
@@ -209,7 +217,8 @@ create_url_from_statement (EphySQLiteStatement *statement)
 
   url->id = ephy_sqlite_statement_get_column_as_int (statement, 0);
   url->host = ephy_history_host_new (NULL, NULL, 0, 1.0);
-  url->host->id = ephy_sqlite_statement_get_column_as_int (statement, 6);
+  url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
+  url->host->id = ephy_sqlite_statement_get_column_as_int (statement, 7);
 
   return url;
 }
@@ -231,6 +240,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
       "urls.visit_count, "
       "urls.typed_count, "
       "urls.last_visit_time, "
+      "urls.hidden_from_overview, "
       "urls.host "
     "FROM "
       "urls ";
@@ -252,6 +262,9 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
     statement_str = g_string_append (statement_str, "WHERE ");
   }
 
+  if (query->ignore_hidden)
+    statement_str = g_string_append (statement_str, "urls.hidden_from_overview = 0 AND ");
+
   if (query->host > 0)
     statement_str = g_string_append (statement_str, "urls.host = ? AND ");
 
@@ -282,7 +295,6 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   if (error) {
     g_error ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
-    g_object_unref (statement);
     return NULL;
   }
 
@@ -311,7 +323,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
     }
   }
   for (substring = query->substring_list; substring != NULL; substring = substring->next) {
-    char *string = g_strdup_printf ("%%%s%%", (char*)substring->data);
+    char *string = ephy_sqlite_create_match_pattern (substring->data);
     if (ephy_sqlite_statement_bind_string (statement, i++, string, &error) == FALSE) {
       g_error ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
@@ -379,7 +391,6 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
   if (error) {
     g_error ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
-    g_object_unref (statement);
     return;
   }
 
