@@ -30,11 +30,16 @@
 #include "ephy-gui.h"
 #include "ephy-signal-accumulator.h"
 
+#include <libsoup/soup.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 /**
  * SECTION:ephy-location-entry
@@ -196,14 +201,28 @@ ephy_location_entry_finalize (GObject *object)
 	G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
 
+static void
+ephy_location_entry_get_preferred_width (GtkWidget       *widget,
+					 gint            *minimum_width,
+					 gint            *natural_width)
+{
+	if (minimum_width)
+		*minimum_width = -1;
+
+	if (natural_width)
+		*natural_width = 518;
+}
+
 static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->get_property = ephy_location_entry_get_property;
 	object_class->set_property = ephy_location_entry_set_property;
 	object_class->finalize = ephy_location_entry_finalize;
+	widget_class->get_preferred_width = ephy_location_entry_get_preferred_width;
 
 	/**
 	* EphyLocationEntry:location:
@@ -574,22 +593,16 @@ entry_populate_popup_cb (GtkEntry *entry,
 			 EphyLocationEntry *lentry)
 {
 	EphyLocationEntryPrivate *priv = lentry->priv;
-	GtkWidget *image;
 	GtkWidget *clear_menuitem, *undo_menuitem, *redo_menuitem, *separator;
 	GList *children, *item;
 	int pos = 0, sep = 0;
 	gboolean is_editable;
 
-	/* Clear and Copy mnemonics conflict, make custom menuitem */
-	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (image);
-
 	/* Translators: the mnemonic shouldn't conflict with any of the
 	 * standard items in the GtkEntry context menu (Cut, Copy, Paste, Delete,
 	 * Select All, Input Methods and Insert Unicode control character.)
 	 */
-	clear_menuitem = gtk_image_menu_item_new_with_mnemonic (_("Cl_ear"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (clear_menuitem), image);
+	clear_menuitem = gtk_menu_item_new_with_mnemonic (_("Cl_ear"));
 	g_signal_connect (clear_menuitem , "activate",
 			  G_CALLBACK (entry_clear_activate_cb), lentry);
 	is_editable = gtk_editable_get_editable (GTK_EDITABLE (entry));
@@ -608,14 +621,14 @@ entry_populate_popup_cb (GtkEntry *entry,
 
 	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), clear_menuitem, pos - 1);
 	
-	undo_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_UNDO, NULL);
+	undo_menuitem = gtk_menu_item_new_with_mnemonic (_("_Undo"));
 	gtk_widget_set_sensitive (undo_menuitem, priv->user_changed);
 	g_signal_connect (undo_menuitem, "activate",
 			  G_CALLBACK (entry_undo_activate_cb), lentry);
 	gtk_widget_show (undo_menuitem);
 	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), undo_menuitem, 0);
 	
-	redo_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_REDO, NULL);
+	redo_menuitem = gtk_menu_item_new_with_mnemonic (_("_Redo"));
 	gtk_widget_set_sensitive (redo_menuitem, priv->can_redo);
 	g_signal_connect (redo_menuitem, "activate",
 			  G_CALLBACK (entry_redo_activate_cb), lentry);
@@ -657,6 +670,7 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	char *title = NULL, *address = NULL;
 	GString *text;
 	GtkStyleContext *style;
+	const PangoFontDescription *font_desc;
 	cairo_surface_t *surface;
 	PangoContext *context;
 	PangoLayout  *layout;
@@ -700,8 +714,10 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	layout = pango_layout_new (context);
 
 	style = gtk_widget_get_style_context (GTK_WIDGET (entry));
+	gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL,
+			       "font", &font_desc, NULL);
 	metrics = pango_context_get_metrics (context,
-		                             gtk_style_context_get_font (style, GTK_STATE_FLAG_NORMAL),
+		                             font_desc,
 					     pango_context_get_language (context));
 
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
@@ -938,11 +954,15 @@ static gboolean
 do_dns_prefetch (PrefetchHelper *helper)
 {
 #ifdef HAVE_WEBKIT2
-	/* TODO: Network Features */
+	WebKitWebContext *context = webkit_web_context_get_default ();
+
+	if (helper->uri)
+		webkit_web_context_prefetch_dns (context, helper->uri->host);
 #else
 	SoupSession *session = webkit_get_default_session ();
 
-	soup_session_prepare_for_uri (session, helper->uri);
+	if (helper->uri)
+		soup_session_prefetch_dns (session, helper->uri->host, NULL, NULL, NULL);
 #endif
 
 	helper->entry->priv->dns_prefetch_handler = 0;
@@ -954,13 +974,20 @@ static void
 schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *url)
 {
 	PrefetchHelper *helper;
+	SoupURI *uri;
+
+	uri = soup_uri_new (url);
+	if (!uri || !uri->host) {
+		soup_uri_free (uri);
+		return;
+	}
 
 	if (entry->priv->dns_prefetch_handler)
 		g_source_remove (entry->priv->dns_prefetch_handler);
 
 	helper = g_slice_new0 (PrefetchHelper);
 	helper->entry = g_object_ref (entry);
-	helper->uri = soup_uri_new (url);
+	helper->uri = uri;
 
 	entry->priv->dns_prefetch_handler =
 		g_timeout_add_full (G_PRIORITY_DEFAULT, interval,
