@@ -33,11 +33,10 @@
 #include "ephy-node-common.h"
 #include "ephy-node-view.h"
 #include "ephy-prefs.h"
-#include "ephy-search-entry.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "ephy-state.h"
+#include "ephy-initial-state.h"
 #include "ephy-topic-action.h"
 #include "ephy-window.h"
 #include "popup-commands.h"
@@ -291,11 +290,7 @@ get_target_window (EphyBookmarksEditor *editor)
 	}
 	else
 	{
-		EphySession *session;
-
-		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-
-		return GTK_WIDGET (ephy_session_get_active_window (session));
+		return GTK_WIDGET (gtk_application_get_active_window (GTK_APPLICATION (ephy_shell_get_default ())));
 	}
 }
 
@@ -319,7 +314,8 @@ cmd_open_bookmarks_in_tabs (GtkAction *action,
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_BMK_PROP_LOCATION);
 
-		new_embed = ephy_shell_new_tab (ephy_shell, window, NULL, location,
+		new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
+						window, NULL, location,
 				 	        EPHY_NEW_TAB_OPEN_PAGE |
 					        EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 		/* if there was no target window, a new one was opened. Get it
@@ -355,8 +351,10 @@ cmd_open_bookmarks_in_browser (GtkAction *action,
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_BMK_PROP_LOCATION);
 
-		ephy_shell_new_tab (ephy_shell, window, NULL, location,
-				    EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_NEW_WINDOW);
+		ephy_shell_new_tab (ephy_shell_get_default (),
+				    window, NULL, location,
+				    EPHY_NEW_TAB_OPEN_PAGE |
+				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
 
 	g_list_free (selection);
@@ -616,7 +614,7 @@ import_dialog_response_cb (GtkDialog *dialog,
 			dialog = ephy_file_chooser_new (_("Import Bookmarks from File"),
 							GTK_WIDGET (editor),
 							GTK_FILE_CHOOSER_ACTION_OPEN,
-							NULL, EPHY_FILE_FILTER_NONE);
+							EPHY_FILE_FILTER_NONE);
 
 			ephy_file_chooser_add_mime_filter
 				(dialog,
@@ -755,7 +753,6 @@ cmd_bookmarks_export (GtkAction *action,
 	dialog = GTK_WIDGET (ephy_file_chooser_new (_("Export Bookmarks"),
 		GTK_WIDGET (editor),
 		GTK_FILE_CHOOSER_ACTION_SAVE,
-		NULL,
 		EPHY_FILE_FILTER_NONE));
 
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
@@ -815,10 +812,10 @@ cmd_bookmarks_import (GtkAction *action,
 	dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
 					      GTK_WINDOW (editor),
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_STOCK_CANCEL,
+					      _("_Cancel"),
 					      GTK_RESPONSE_CANCEL,
 					      _("I_mport"),
-					      GTK_RESPONSE_OK,
+					      _("_OK"),
 					      NULL);
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
@@ -979,7 +976,7 @@ static void
 cmd_help_contents (GtkAction *action,
 		   EphyBookmarksEditor *editor)
 {
-	ephy_gui_help (GTK_WIDGET (editor), "ephy-managing-bookmarks");
+	ephy_gui_help (GTK_WIDGET (editor), "bookmark");
 }
 
 static void
@@ -1049,7 +1046,8 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
 
-	ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    NULL, NULL, location,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
@@ -1067,7 +1065,8 @@ ephy_bookmarks_editor_node_middle_clicked_cb (GtkWidget *view,
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
 
-	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    window, NULL, location,
 			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 }
@@ -1139,9 +1138,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		key_normal = (priority == EPHY_NODE_NORMAL_PRIORITY);
 
 		EPHY_TOPIC_ACTION_NAME_PRINTF (name, node);
-
-		g_list_free (selected);
 	}
+	g_list_free (selected);
 
 	selected = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->bm_view));
 	if (bmk_focus && selected)
@@ -1154,9 +1152,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		mutable = !ephy_node_get_property_boolean (node, EPHY_NODE_BMK_PROP_IMMUTABLE);
 		
 		EPHY_BOOKMARK_ACTION_NAME_PRINTF (name, node);
-
-		g_list_free (selected);
 	}
+	g_list_free (selected);
 
 	open_in_window_label = ngettext ("Open in New _Window",
 					 "Open in New _Windows",
@@ -1330,7 +1327,7 @@ keyword_node_selected_cb (EphyNodeView *view,
 	}
 	else
 	{
-		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
+		gtk_entry_set_text (GTK_ENTRY (editor->priv->search_entry), "");
 		bookmarks_filter (editor, node);
 	}
 }
@@ -1348,9 +1345,10 @@ keyword_node_show_popup_cb (GtkWidget *view, EphyBookmarksEditor *editor)
 }
 
 static void
-search_entry_search_cb (GtkWidget *entry, const char *search_text, EphyBookmarksEditor *editor)
+search_entry_changed_cb (GtkWidget *entry, EphyBookmarksEditor *editor)
 {
 	EphyNode *all;
+	const char *search_text;
 
 	g_signal_handlers_block_by_func
 		(G_OBJECT (editor->priv->key_view),
@@ -1364,6 +1362,8 @@ search_entry_search_cb (GtkWidget *entry, const char *search_text, EphyBookmarks
 		 G_CALLBACK (keyword_node_selected_cb),
 		 editor);
 
+	search_text = gtk_entry_get_text (GTK_ENTRY (entry));
+
 	ephy_node_filter_empty (editor->priv->bookmarks_filter);
 	ephy_node_filter_add_expression (editor->priv->bookmarks_filter,
 					 ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
@@ -1387,35 +1387,23 @@ static GtkWidget *
 build_search_box (EphyBookmarksEditor *editor)
 {
 	GtkWidget *box;
-	GtkWidget *label;
 	GtkWidget *entry;
-	char *str;
 
 	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 	gtk_widget_show (box);
 
-	entry = ephy_search_entry_new ();
+	entry = gtk_search_entry_new ();
 	editor->priv->search_entry = entry;
 	
-	g_signal_connect (G_OBJECT (entry), "search",
-			  G_CALLBACK (search_entry_search_cb),
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (search_entry_changed_cb),
 			  editor);
 	add_entry_monitor (editor, entry);
 	add_focus_monitor (editor, entry);
 	
 	gtk_widget_show_all (entry);
 
-	label = gtk_label_new (NULL);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	str = g_strconcat ("<b>", _("_Search:"), "</b>", NULL);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
-	g_free (str);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-	gtk_widget_show (label);
-
-	gtk_box_pack_start (GTK_BOX (box),
-			    label, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (box),
 			    entry, TRUE, TRUE, 0);
 
@@ -1460,27 +1448,61 @@ node_dropped_cb (EphyNodeView *view,
 	}
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Favicons */
+#else
+static gboolean
+webkit_favicon_database_has_favicon (WebKitFaviconDatabase *database, const char *page_uri)
+{
+    gboolean result;
+    char *uri;
+
+    uri = webkit_favicon_database_get_favicon_uri (database, page_uri);
+    result = (uri != NULL);
+    g_free (uri);
+
+    return result;
+}
+
 static void
-icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GValue *value)
+icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GtkTreeRowReference *reference)
 {
     GdkPixbuf *favicon = webkit_favicon_database_get_favicon_pixbuf_finish (database, result, NULL);
-    if (!favicon)
-      return;
 
-    g_value_take_object (value, favicon);
+    if (favicon && gtk_tree_row_reference_valid (reference)) {
+	    GtkTreeModel *model = gtk_tree_row_reference_get_model (reference);
+	    GtkTreePath *path = gtk_tree_row_reference_get_path (reference);
+	    GtkTreeIter iter;
+
+	    /* Force repaint. */
+	    if (gtk_tree_model_get_iter (model, &iter, path))
+		    gtk_tree_model_row_changed (model, path, &iter);
+
+	    gtk_tree_path_free (path);
+    }
+
+    gtk_tree_row_reference_free (reference);
+    if (favicon)
+	    g_object_unref (favicon);
 }
+#endif
 
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
-	const char *page_location;
 	GdkPixbuf *favicon = NULL;
+#ifndef HAVE_WEBKIT2
+	const char *page_location;
 
 	page_location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 
 	LOG ("Get favicon for %s", page_location ? page_location : "None");
+#endif
 
+#ifdef HAVE_WEBKIT2
+        /* TODO: Favicons */
+#else
 	if (page_location)
         {
 		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
@@ -1489,12 +1511,26 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
                 favicon = webkit_favicon_database_try_get_favicon_pixbuf (database, page_location,
 									  FAVICON_SIZE, FAVICON_SIZE);
 
-		if (!favicon && webkit_favicon_database_get_favicon_uri (database, page_location))
-		  webkit_favicon_database_get_favicon_pixbuf (database, page_location,
-							      FAVICON_SIZE, FAVICON_SIZE, NULL,
-							      (GAsyncReadyCallback) icon_loaded_cb, value);
-        }
+		if (!favicon && webkit_favicon_database_has_favicon (database, page_location)) {
+			GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (user_data));
+			GtkTreeIter iter;
 
+			if (ephy_node_view_get_iter_for_node (EPHY_NODE_VIEW (user_data), &iter, node)) {
+				GtkTreeRowReference *reference;
+				GtkTreePath *path;
+
+				path = gtk_tree_model_get_path (model, &iter);
+				reference = gtk_tree_row_reference_new (model, path);
+				gtk_tree_path_free (path);
+
+				webkit_favicon_database_get_favicon_pixbuf (database, page_location,
+									    FAVICON_SIZE, FAVICON_SIZE, NULL,
+									    (GAsyncReadyCallback) icon_loaded_cb,
+									    reference);
+			}
+		}
+        }
+#endif
 	g_value_init (value, GDK_TYPE_PIXBUF);
 	g_value_take_object (value, favicon);
 }
@@ -1553,9 +1589,9 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
-	gtk_ui_manager_add_ui_from_file (ui_merge,
-					 ephy_file ("epiphany-bookmark-editor-ui.xml"),
-					 NULL);
+	gtk_ui_manager_add_ui_from_resource (ui_merge,
+                                             "/org/gnome/epiphany/epiphany-bookmark-editor-ui.xml",
+                                             NULL);
 	gtk_window_add_accel_group (GTK_WINDOW (editor), 
 				    gtk_ui_manager_get_accel_group (ui_merge));
 	gtk_ui_manager_ensure_update (ui_merge);
@@ -1730,13 +1766,13 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			  G_CALLBACK (view_selection_changed_cb),
 			  editor);
 
-	ephy_state_add_window (GTK_WIDGET(editor),
-			       "bookmarks_editor",
-			       450, 400, FALSE,
-			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
-	ephy_state_add_paned  (GTK_WIDGET (hpaned),
-			       "bookmarks_paned",
-			       130);
+	ephy_initial_state_add_window (GTK_WIDGET(editor),
+                                       "bookmarks_editor",
+                                       450, 400, FALSE,
+                                       EPHY_INITIAL_STATE_WINDOW_SAVE_SIZE | EPHY_INITIAL_STATE_WINDOW_SAVE_POSITION);
+	ephy_initial_state_add_paned  (GTK_WIDGET (hpaned),
+                                       "bookmarks_paned",
+                                       130);
 
 	/* Lockdown settings */
 	action = gtk_action_group_get_action (action_group, "Export");
