@@ -25,28 +25,30 @@
 #include "ephy-debug.h"
 #include "ephy-embed-utils.h"
 
+#include <math.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
-#define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
+#define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
 
 struct _EphyFindToolbarPrivate
 {
 	EphyWindow *window;
 	WebKitWebView *web_view;
+#ifdef HAVE_WEBKIT2
+        WebKitFindController *controller;
+#endif
 	GtkWidget *entry;
-	GtkWidget *label;
-	GtkToolItem *next;
-	GtkToolItem *prev;
-	GtkToolItem *sep;
-	GtkToolItem *status_item;
-	GtkWidget *status_label;
-	GtkWidget *case_sensitive;
-	gulong set_focus_handler;
-	guint source_id;
+	GtkWidget *next;
+	GtkWidget *prev;
 	guint find_again_source_id;
 	guint find_source_id;
 	char *find_string;
@@ -86,12 +88,17 @@ typedef enum
 
 static guint signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, GTK_TYPE_SEARCH_BAR)
+
 /* private functions */
 
 static void
 scroll_lines (WebKitWebView *web_view,
               int num_lines)
 {
+#ifdef HAVE_WEBKIT2
+        /* TODO: Scroll API? */
+#else
         GtkScrolledWindow *scrolled_window;
         GtkAdjustment *vadj;
         gdouble value;
@@ -101,12 +108,16 @@ scroll_lines (WebKitWebView *web_view,
 
         value = gtk_adjustment_get_value (vadj) + (num_lines * gtk_adjustment_get_step_increment (vadj));
         gtk_adjustment_set_value (vadj, value);
+#endif
 }
 
 static void
 scroll_pages (WebKitWebView *web_view,
               int num_pages)
 {
+#ifdef HAVE_WEBKIT2
+        /* TODO: Scroll API */
+#else
         GtkScrolledWindow *scrolled_window;
         GtkAdjustment *vadj;
         gdouble value;
@@ -116,23 +127,7 @@ scroll_pages (WebKitWebView *web_view,
 
         value = gtk_adjustment_get_value (vadj) + (num_pages * gtk_adjustment_get_page_increment (vadj));
         gtk_adjustment_set_value (vadj, value);
-}
-
-static gboolean
-set_status_notfound_cb (EphyFindToolbar *toolbar)
-{
-	EphyFindToolbarPrivate *priv;
-	PangoFontDescription *pango_desc = NULL;
-
-	priv = toolbar->priv;
-
-	pango_desc = pango_font_description_new ();
-	gtk_widget_override_font (priv->status_label, pango_desc);
-	pango_font_description_free (pango_desc);
-
-	priv->source_id = 0;
-
-	return FALSE;
+#endif
 }
 
 static void
@@ -140,37 +135,36 @@ set_status (EphyFindToolbar *toolbar,
 	    EphyFindResult result)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
-	char *text = NULL;
-	PangoFontDescription *pango_desc = NULL;
+	const char *icon_name = "edit-find-symbolic";
+	const char *tooltip = NULL;
+	gboolean rtl;
 
 	switch (result)
 	{
 		case EPHY_FIND_RESULT_FOUND:
-			text = NULL;
 			break;
 		case EPHY_FIND_RESULT_NOTFOUND:
-			{
-				text = _("Not found");
+			icon_name = "face-uncertain-symbolic";
+			tooltip = _("Text not found");
+			gtk_widget_error_bell (GTK_WIDGET (priv->window));
 
-				pango_desc = pango_font_description_new ();
-				pango_font_description_set_weight (pango_desc, PANGO_WEIGHT_BOLD);
-				gtk_widget_override_font (priv->status_label, pango_desc);
-				pango_font_description_free (pango_desc);
-
-				gtk_widget_error_bell (GTK_WIDGET (priv->window));
-				priv->source_id = g_timeout_add (500, (GSourceFunc) set_status_notfound_cb, toolbar);
-			}
 			break;
 		case EPHY_FIND_RESULT_FOUNDWRAPPED:
-			text = _("Wrapped");
+			rtl = gtk_widget_get_direction (GTK_WIDGET (toolbar)) == GTK_TEXT_DIR_RTL;
+			icon_name = rtl ? "view-wrapped-rtl-symbolic" : "view-wrapped-symbolic";
+			tooltip = _("Search wrapped back to the top");
 			break;
 	}
 
-	gtk_label_set_text (GTK_LABEL (priv->status_label),
-			    text != NULL ? text : "");
+	gtk_widget_set_sensitive (priv->prev, result != EPHY_FIND_RESULT_NOTFOUND);
+	gtk_widget_set_sensitive (priv->next, result != EPHY_FIND_RESULT_NOTFOUND);
 
-	g_object_set (priv->sep, "visible", text != NULL, NULL);
-	g_object_set (priv->status_item, "visible", text != NULL, NULL);
+	g_object_set (priv->entry,
+		      "primary-icon-name", icon_name,
+		      "primary-icon-activatable", FALSE,
+		      "primary-icon-sensitive", FALSE,
+		      "primary-icon-tooltip-text", tooltip,
+		      NULL);
 }
 
 static void
@@ -178,11 +172,19 @@ clear_status (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	gtk_widget_hide (GTK_WIDGET (priv->sep));
-	gtk_widget_hide (GTK_WIDGET (priv->status_item));
-	gtk_label_set_text (GTK_LABEL (priv->status_label), "");
-	gtk_label_set_text (GTK_LABEL (priv->label),
-			    priv->links_only ? _("Find links:") : _("Find:"));
+	g_object_set (priv->entry,
+		      "primary-icon-name", "edit-find-symbolic",
+		      NULL);
+
+	gtk_widget_set_sensitive (priv->prev, FALSE);
+	gtk_widget_set_sensitive (priv->next, FALSE);
+
+        if (priv->web_view == NULL) return;
+#ifdef HAVE_WEBKIT2
+        webkit_find_controller_search_finish (priv->controller);
+#else
+	webkit_web_view_unmark_text_matches (priv->web_view);
+#endif
 }
 
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
@@ -198,7 +200,7 @@ tab_search_key_press_cb (EphyEmbed *embed,
 	g_return_val_if_fail (event != NULL, FALSE);
 
 	/* check for / and ' which open the find toolbar in text resp. link mode */
-	if (gtk_widget_get_visible (widget) == FALSE)
+	if (gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (widget)) == FALSE)
 	{
 		if (event->keyval == GDK_KEY_slash)
 		{
@@ -227,6 +229,24 @@ find_prev_cb (EphyFindToolbar *toolbar)
 	g_signal_emit (toolbar, signals[PREVIOUS], 0);
 }
 
+static gboolean
+str_has_uppercase (const char *str)
+{
+	while (str != NULL && *str != '\0') {
+		gunichar c;
+
+		c = g_utf8_get_char (str);
+
+		if (g_unichar_isupper (c))
+			return TRUE;
+
+		str = g_utf8_next_char (str);
+	}
+
+	return FALSE;
+}
+
+#ifndef HAVE_WEBKIT2
 static void
 ephy_find_toolbar_mark_matches (EphyFindToolbar *toolbar)
 {
@@ -234,7 +254,7 @@ ephy_find_toolbar_mark_matches (EphyFindToolbar *toolbar)
         WebKitWebView *web_view = priv->web_view;
         gboolean case_sensitive;
 
-        case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+        case_sensitive = str_has_uppercase (priv->find_string);
 
         webkit_web_view_unmark_text_matches (web_view);
         if (priv->find_string != NULL && priv->find_string[0] != '\0')
@@ -244,15 +264,36 @@ ephy_find_toolbar_mark_matches (EphyFindToolbar *toolbar)
                                                    0);
         webkit_web_view_set_highlight_text_matches (web_view, TRUE);
 }
+#endif
 
+#ifdef HAVE_WEBKIT2
+static void
+real_find (EphyFindToolbarPrivate *priv,
+           EphyFindDirection direction)
+{
+        WebKitFindOptions options = WEBKIT_FIND_OPTIONS_NONE;
+
+        if (!g_strcmp0 (priv->find_string, ""))
+                return;
+
+        if (!str_has_uppercase (priv->find_string))
+                options |= WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+        if (direction == EPHY_FIND_DIRECTION_PREV)
+                options |= WEBKIT_FIND_OPTIONS_BACKWARDS;
+
+        webkit_find_controller_search (priv->controller, priv->find_string, options, G_MAXUINT);
+}
+
+#else
 static EphyFindResult
 real_find (EphyFindToolbarPrivate *priv,
 	   EphyFindDirection direction)
 {
         WebKitWebView *web_view = priv->web_view;
-        gboolean case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+        gboolean case_sensitive;
         gboolean forward = (direction == EPHY_FIND_DIRECTION_NEXT);
 
+        case_sensitive = str_has_uppercase (priv->find_string);
         if (!priv->find_string || !g_strcmp0 (priv->find_string, ""))
                 return EPHY_FIND_RESULT_NOTFOUND;
 
@@ -271,26 +312,66 @@ real_find (EphyFindToolbarPrivate *priv,
 
         return EPHY_FIND_RESULT_FOUND;
 }
+#endif
 
 static gboolean
 do_search (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
+#ifndef HAVE_WEBKIT2
 	EphyFindResult result;
+#endif
 
 	priv->find_source_id = 0;
 
+#ifdef HAVE_WEBKIT2
+        real_find (priv, EPHY_FIND_DIRECTION_NEXT);
+#else
 	ephy_find_toolbar_mark_matches (toolbar);
 
 	result = real_find (priv, EPHY_FIND_DIRECTION_NEXT);
 	set_status (toolbar, result);
+#endif
 
 	return FALSE;
 }
 
+#ifdef HAVE_WEBKIT2
 static void
-entry_changed_cb (GtkEntry *entry,
-		  EphyFindToolbar *toolbar)
+found_text_cb (WebKitFindController *controller,
+               guint n_matches,
+               EphyFindToolbar *toolbar)
+{
+        WebKitFindOptions options;
+        EphyFindResult result;
+
+        options = webkit_find_controller_get_options (controller);
+        /* FIXME: it's not possible to remove the wrap flag, so the status is now always wrapped. */
+        result = options & WEBKIT_FIND_OPTIONS_WRAP_AROUND ? EPHY_FIND_RESULT_FOUNDWRAPPED : EPHY_FIND_RESULT_FOUND;
+        set_status (toolbar, result);
+}
+
+static void
+failed_to_find_text_cb (WebKitFindController *controller,
+                        EphyFindToolbar *toolbar)
+{
+        EphyFindToolbarPrivate *priv = toolbar->priv;
+        WebKitFindOptions options;
+
+        options = webkit_find_controller_get_options (controller);
+        if (options & WEBKIT_FIND_OPTIONS_WRAP_AROUND) {
+                set_status (toolbar, EPHY_FIND_RESULT_NOTFOUND);
+                return;
+        }
+
+        options |= WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+        webkit_find_controller_search (controller, priv->find_string, options, G_MAXUINT);
+}
+#endif
+
+
+static void
+update_find_string (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
@@ -302,8 +383,10 @@ entry_changed_cb (GtkEntry *entry,
 		priv->find_source_id = 0;
 	}
 
-	if (strlen (priv->find_string) == 0)
-                return;
+	if (strlen (priv->find_string) == 0) {
+		clear_status (toolbar);
+		return;
+	}
 
 	priv->find_source_id = g_timeout_add (300, (GSourceFunc)do_search, toolbar);
 }
@@ -389,106 +472,6 @@ entry_activate_cb (GtkWidget *entry,
 	}
 }
 
-static void
-case_sensitive_menu_toggled_cb (GtkWidget *check,
-				EphyFindToolbar *toolbar)
-{
-	EphyFindToolbarPrivate *priv = toolbar->priv;
-	gboolean case_sensitive;
-
-	case_sensitive = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (check));
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->case_sensitive),
-				      case_sensitive);
-}
-
-static void
-case_sensitive_toggled_cb (GtkWidget *check,
-			   EphyFindToolbar *toolbar)
-{
-	gboolean case_sensitive;
-	GtkWidget *proxy;
-
-	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
-
-	proxy = gtk_tool_item_get_proxy_menu_item (GTK_TOOL_ITEM (gtk_widget_get_parent (check)),
-						   "menu-proxy");
-
-	if (proxy != NULL)
-	{
-		g_signal_handlers_block_by_func
-			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
-
-		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (proxy),
-						case_sensitive);
-		g_signal_handlers_unblock_by_func
-			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
-	}
-
-	ephy_find_toolbar_mark_matches (toolbar);
-
-	/*
-	 * If we now use the stricter method (and are case sensitive),
-	 * check that the current selection still matches. If not, find the
-	 * next one.
-	 * This currently requires going back and then forward, because
-	 * there's no function in WebKit that would verify the current selection.
-	 */
-	if (case_sensitive)
-	{
-		EphyFindResult result;
-
-		result = real_find (toolbar->priv, EPHY_FIND_DIRECTION_PREV);
-		if (result != EPHY_FIND_RESULT_NOTFOUND)
-			result = real_find (toolbar->priv,
-					    EPHY_FIND_DIRECTION_NEXT);
-
-		set_status (toolbar, result);
-	}
-}
-
-static gboolean
-toolitem_create_menu_proxy_cb (GtkToolItem *toolitem,
-			       EphyFindToolbar *toolbar)
-{
-	EphyFindToolbarPrivate *priv = toolbar->priv;
-	GtkWidget *checkbox_menu;
-	gboolean case_sensitive;
-
-	/* Create a menu item, and sync it */
-	checkbox_menu = gtk_check_menu_item_new_with_mnemonic (_("_Case sensitive"));
-	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (checkbox_menu),
-					case_sensitive);
-
-	g_signal_connect (checkbox_menu, "toggled",
-			  G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
-
-	gtk_tool_item_set_proxy_menu_item (toolitem, "menu-proxy",
-					   checkbox_menu);
-
-	return TRUE;
-}
-
-static void
-set_focus_cb (EphyWindow *window,
-	      GtkWidget *widget,
-	      EphyFindToolbar *toolbar)
-{
-	GtkWidget *wtoolbar = GTK_WIDGET (toolbar);
-
-	while (widget != NULL && widget != wtoolbar)
-	{
-		widget = gtk_widget_get_parent (widget);
-	}
-
-	/* if widget == toolbar, the new focus widget is in the toolbar */
-	if (widget != wtoolbar)
-	{
-		ephy_find_toolbar_request_close (toolbar);
-	}
-}
-
 static void
 ephy_find_toolbar_set_window (EphyFindToolbar *toolbar,
 			      EphyWindow *window)
@@ -498,126 +481,144 @@ ephy_find_toolbar_set_window (EphyFindToolbar *toolbar,
 	priv->window = window;
 }
 
-static void
-ephy_find_toolbar_parent_set (GtkWidget *widget,
-			      GtkWidget *previous_parent)
-{
-	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (widget);
-	EphyFindToolbarPrivate *priv = toolbar->priv;
-	GtkWidget *toplevel;
-
-	if (gtk_widget_get_parent (widget) != NULL &&
-	    priv->set_focus_handler == 0)
-	{
-		toplevel = gtk_widget_get_toplevel (widget);
-		priv->set_focus_handler =
-			g_signal_connect (toplevel, "set-focus",
-					  G_CALLBACK (set_focus_cb), toolbar);
-	}
-}
-
 static void
 ephy_find_toolbar_grab_focus (GtkWidget *widget)
 {
 	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (widget);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
+	gtk_widget_grab_focus (priv->entry);
+}
+
+static gboolean
+ephy_find_toolbar_draw (GtkWidget *widget,
+			cairo_t *cr)
+{
+	GtkStyleContext *context;
+
+	context = gtk_widget_get_style_context (widget);
+
+	gtk_style_context_save (context);
+	gtk_style_context_set_state (context, gtk_widget_get_state_flags (widget));
+
+	gtk_render_background (context, cr, 0, 0,
+			       gtk_widget_get_allocated_width (widget),
+			       gtk_widget_get_allocated_height (widget));
+
+	gtk_render_frame (context, cr, 0, 0,
+			  gtk_widget_get_allocated_width (widget),
+			  gtk_widget_get_allocated_height (widget));
+
+	gtk_style_context_restore (context);
+
+	return GTK_WIDGET_CLASS (ephy_find_toolbar_parent_class)->draw (widget, cr);
+}
+
+static void
+search_entry_clear_cb (GtkEntry *entry,
+                       gpointer  user_data)
+{
+  gtk_entry_set_text (entry, "");
+}
+
+static void
+search_entry_changed_cb (GtkEntry *entry,
+                         EphyFindToolbar *toolbar)
+{
+        const char *str;
+        const char *primary_icon_name = "edit-find-symbolic";
+        const char *secondary_icon_name = NULL;
+        gboolean primary_active = FALSE;
+        gboolean secondary_active = FALSE;
+
+        str = gtk_entry_get_text (entry);
+
+        if (str == NULL || *str == '\0') {
+                primary_icon_name = "edit-find-symbolic";
+        } else {
+                if (gtk_widget_get_direction (GTK_WIDGET (entry)) == GTK_TEXT_DIR_RTL)
+                        secondary_icon_name = "edit-clear-rtl-symbolic";
+                else
+                        secondary_icon_name = "edit-clear-symbolic";
+                secondary_active = TRUE;
+        }
+
+        g_object_set (entry,
+                      "primary-icon-name", primary_icon_name,
+                      "primary-icon-activatable", primary_active,
+                      "primary-icon-sensitive", primary_active,
+                      "secondary-icon-name", secondary_icon_name,
+                      "secondary-icon-activatable", secondary_active,
+                      "secondary-icon-sensitive", secondary_active,
+                      NULL);
+
+        update_find_string (toolbar);
 }
 
 static void
 ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv;
-	GtkToolbar *gtoolbar;
-	GtkToolItem *item;
-	GtkWidget *alignment, *arrow, *box;
-	GtkWidget *checkbox;
+	GtkWidget *box;
+	GtkSizeGroup *size_group;
 
 	priv = toolbar->priv = EPHY_FIND_TOOLBAR_GET_PRIVATE (toolbar);
-	gtoolbar = GTK_TOOLBAR (toolbar);
 
-	gtk_toolbar_set_style (gtoolbar, GTK_TOOLBAR_BOTH_HORIZ);
+	size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 
-	/* Find: |_____| */
-	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 2, 2);
-
-	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-	gtk_container_add (GTK_CONTAINER (alignment), box);
-
-	priv->label = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (box), priv->label, FALSE, FALSE, 0);
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_style_context_add_class (gtk_widget_get_style_context (box),
+				     GTK_STYLE_CLASS_RAISED);
+	gtk_style_context_add_class (gtk_widget_get_style_context (box),
+				     GTK_STYLE_CLASS_LINKED);
+	gtk_container_add (GTK_CONTAINER (toolbar), box);
 
 	priv->entry = gtk_entry_new ();
 	gtk_entry_set_width_chars (GTK_ENTRY (priv->entry), 32);
 	gtk_entry_set_max_length (GTK_ENTRY (priv->entry), 512);
-	gtk_box_pack_start (GTK_BOX (box), priv->entry, TRUE, TRUE, 0);
-
-	item = gtk_tool_item_new ();
-	gtk_container_add (GTK_CONTAINER (item), alignment);
-	/* gtk_tool_item_set_expand (item, TRUE); */
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
-	gtk_widget_show_all (GTK_WIDGET (item));
+	gtk_entry_set_placeholder_text (GTK_ENTRY (priv->entry), _("Type to search…"));
+	gtk_container_add (GTK_CONTAINER (box), priv->entry);
 
 	/* Prev */
-	arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
-	priv->prev = gtk_tool_button_new (arrow, _("Find Previous"));
-	gtk_tool_item_set_is_important (priv->prev, TRUE);
-	gtk_tool_item_set_tooltip_text (priv->prev,
-					_("Find previous occurrence of the search string"));
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->prev, -1);
-	gtk_widget_show_all (GTK_WIDGET (priv->prev));
+	priv->prev = gtk_button_new_from_icon_name ("go-up-symbolic", GTK_ICON_SIZE_MENU);
+	gtk_widget_set_tooltip_text (priv->prev,
+				     _("Find previous occurrence of the search string"));
+	gtk_container_add (GTK_CONTAINER (box), priv->prev);
+	gtk_widget_show_all (priv->prev);
+	gtk_widget_set_sensitive (priv->prev, FALSE);
 
 	/* Next */
-	arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
-	priv->next = gtk_tool_button_new (arrow, _("Find Next"));
-	gtk_tool_item_set_is_important (priv->next, TRUE);
-	gtk_tool_item_set_tooltip_text (priv->next,
-					_("Find next occurrence of the search string"));
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->next, -1);
-	gtk_widget_show_all (GTK_WIDGET (priv->next));
+	priv->next = gtk_button_new_from_icon_name ("go-down-symbolic", GTK_ICON_SIZE_MENU);
+	gtk_widget_set_tooltip_text (priv->next,
+				     _("Find next occurrence of the search string"));
+	gtk_container_add (GTK_CONTAINER (box), priv->next);
+	gtk_widget_set_sensitive (priv->next, FALSE);
 
-	/* Case sensitivity */
-	checkbox = gtk_check_button_new_with_mnemonic (_("_Case sensitive"));
-	priv->case_sensitive = checkbox;
-	item = gtk_tool_item_new ();
-	gtk_container_add (GTK_CONTAINER (item), checkbox);
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
-	gtk_widget_show_all (GTK_WIDGET (item));
-
-	/* Populate the overflow menu */
-	g_signal_connect (item, "create-menu-proxy",
-			  G_CALLBACK (toolitem_create_menu_proxy_cb), toolbar);
-
-	priv->sep = gtk_separator_tool_item_new ();
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->sep, -1);
-
-	priv->status_item = gtk_tool_item_new ();
-	gtk_tool_item_set_expand (priv->status_item, TRUE);
-	priv->status_label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (priv->status_label), 0.0, 0.5);
-	gtk_label_set_ellipsize (GTK_LABEL (priv->status_label), PANGO_ELLIPSIZE_END);
-	gtk_container_add (GTK_CONTAINER (priv->status_item), priv->status_label);
-	gtk_widget_show (priv->status_label);
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->status_item, -1);
+	gtk_size_group_add_widget (size_group, priv->entry);
+	gtk_size_group_add_widget (size_group, priv->next);
+	gtk_size_group_add_widget (size_group, priv->prev);
+	g_object_unref (size_group);
 
 	/* connect signals */
+	g_signal_connect (priv->entry, "icon-release",
+			  G_CALLBACK (search_entry_clear_cb), toolbar);
 	g_signal_connect (priv->entry, "key-press-event",
 			  G_CALLBACK (entry_key_press_event_cb), toolbar);
 	g_signal_connect_after (priv->entry, "changed",
-				G_CALLBACK (entry_changed_cb), toolbar);
+				G_CALLBACK (search_entry_changed_cb), toolbar);
 	g_signal_connect (priv->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), toolbar);
 	g_signal_connect_swapped (priv->next, "clicked",
 				  G_CALLBACK (find_next_cb), toolbar);
 	g_signal_connect_swapped (priv->prev, "clicked",
 				  G_CALLBACK (find_prev_cb), toolbar);
-	g_signal_connect (priv->case_sensitive, "toggled",
-			  G_CALLBACK (case_sensitive_toggled_cb), toolbar);
-}
+	gtk_search_bar_connect_entry (GTK_SEARCH_BAR (toolbar),
+				      GTK_ENTRY (priv->entry));
 
-G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, GTK_TYPE_TOOLBAR)
+	search_entry_changed_cb (GTK_ENTRY (priv->entry), toolbar);
+
+	gtk_widget_show_all (GTK_WIDGET (toolbar));
+}
 
 static void
 ephy_find_toolbar_dispose (GObject *object)
@@ -625,12 +626,6 @@ ephy_find_toolbar_dispose (GObject *object)
 	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (object);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	if (priv->source_id != 0)
-	{
-		g_source_remove (priv->source_id);
-		priv->source_id = 0;
-	}
-
 	if (priv->find_again_source_id != 0)
 	{
 		g_source_remove (priv->find_again_source_id);
@@ -693,7 +688,7 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 	object_class->get_property = ephy_find_toolbar_get_property;
 	object_class->set_property = ephy_find_toolbar_set_property;
 
-	widget_class->parent_set = ephy_find_toolbar_parent_set;
+	widget_class->draw = ephy_find_toolbar_draw;
 	widget_class->grab_focus = ephy_find_toolbar_grab_focus;
 
 	klass->next = ephy_find_toolbar_find_next;
@@ -743,9 +738,9 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 EphyFindToolbar *
 ephy_find_toolbar_new (EphyWindow *window)
 {
-	return EPHY_FIND_TOOLBAR (g_object_new (EPHY_TYPE_FIND_TOOLBAR,
-						"window", window,
-						NULL));
+	return g_object_new (EPHY_TYPE_FIND_TOOLBAR,
+			     "window", window,
+			     NULL);
 }
 
 const char *
@@ -767,6 +762,12 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 
 	if (priv->web_view != NULL)
 	{
+#ifdef HAVE_WEBKIT2
+                g_signal_handlers_disconnect_matched (priv->controller,
+                                                      G_SIGNAL_MATCH_DATA,
+                                                      0, 0, NULL, NULL, toolbar);
+#endif
+
                 g_signal_handlers_disconnect_matched (EPHY_WEB_VIEW (web_view),
                                                       G_SIGNAL_MATCH_DATA,
                                                       0, 0, NULL, NULL, toolbar);
@@ -775,6 +776,16 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	priv->web_view = web_view;
 	if (web_view != NULL)
 	{
+#ifdef HAVE_WEBKIT2
+                priv->controller = webkit_web_view_get_find_controller (web_view);
+                g_signal_connect_object (priv->controller, "found-text",
+                                         G_CALLBACK (found_text_cb),
+                                         toolbar, 0);
+                g_signal_connect_object (priv->controller, "failed-to-find-text",
+                                         G_CALLBACK (failed_to_find_text_cb),
+                                         toolbar, 0);
+#endif
+
 		clear_status (toolbar);
 
 		g_signal_connect_object (EPHY_WEB_VIEW (web_view), "search-key-press",
@@ -783,6 +794,7 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	}
 }
 
+#ifndef HAVE_WEBKIT2
 typedef struct
 {
 	EphyFindToolbar *toolbar;
@@ -846,17 +858,26 @@ find_again (EphyFindToolbar *toolbar, EphyFindDirection direction)
 						      data,
 						      (GDestroyNotify) find_again_data_destroy_cb);
 }
+#endif
 
 void
 ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 {
+#ifdef HAVE_WEBKIT2
+        webkit_find_controller_search_next (toolbar->priv->controller);
+#else
 	find_again (toolbar, EPHY_FIND_DIRECTION_NEXT);
+#endif
 }
 
 void
 ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 {
+#ifdef HAVE_WEBKIT2
+        webkit_find_controller_search_previous (toolbar->priv->controller);
+#else
 	find_again (toolbar, EPHY_FIND_DIRECTION_PREV);
+#endif
 }
 
 void
@@ -875,8 +896,9 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
 
-	gtk_widget_show (GTK_WIDGET (toolbar));
-	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
+	gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (toolbar), TRUE);
+	gtk_search_bar_set_show_close_button (GTK_SEARCH_BAR (toolbar), TRUE);
+	gtk_widget_grab_focus (priv->entry);
 }
 
 void
@@ -884,17 +906,20 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	gtk_widget_hide (GTK_WIDGET (toolbar));
+	gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (toolbar), FALSE);
 
 	if (priv->web_view == NULL) return;
-
+#ifdef HAVE_WEBKIT2
+        webkit_find_controller_search_finish (priv->controller);
+#else
 	webkit_web_view_set_highlight_text_matches (priv->web_view, FALSE);
+#endif
 }
 
 void
 ephy_find_toolbar_request_close (EphyFindToolbar *toolbar)
 {
-	if (gtk_widget_get_visible (GTK_WIDGET (toolbar)))
+	if (gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (toolbar)))
 	{
 		g_signal_emit (toolbar, signals[CLOSE], 0);
 	}
