@@ -31,11 +31,10 @@
 #include "ephy-hosts-store.h"
 #include "ephy-hosts-view.h"
 #include "ephy-prefs.h"
-#include "ephy-search-entry.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "ephy-state.h"
+#include "ephy-initial-state.h"
 #include "ephy-time-helpers.h"
 #include "ephy-urls-store.h"
 #include "ephy-urls-view.h"
@@ -90,8 +89,7 @@ static void cmd_select_all		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_help_contents		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void search_entry_search_cb	  (GtkWidget *entry,
-					   char *search_text,
+static void search_entry_changed_cb	  (GtkWidget *entry,
 					   EphyHistoryWindow *editor);
 static void
 filter_now (EphyHistoryWindow *editor, gboolean hosts, gboolean pages);
@@ -101,6 +99,7 @@ filter_now (EphyHistoryWindow *editor, gboolean hosts, gboolean pages);
 struct _EphyHistoryWindowPrivate
 {
 	EphyHistoryService *history_service;
+	GCancellable *cancellable;
 	GtkWidget *hosts_view;
 	GtkWidget *pages_view;
 	EphyURLsStore *urls_store;
@@ -210,7 +209,7 @@ confirmation_dialog_response_cb (GtkWidget *dialog,
 static GtkWidget *
 confirmation_dialog_construct (EphyHistoryWindow *editor)
 {
-	GtkWidget *dialog, *button, *image;
+	GtkWidget *dialog, *button;
 
 	dialog = gtk_message_dialog_new
 		(GTK_WINDOW (editor),
@@ -228,9 +227,6 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 				     GTK_WINDOW (dialog));
 	
 	button = gtk_button_new_with_mnemonic (_("Cl_ear"));
-	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image (GTK_BUTTON (button), image);
-	/* don't show the image! see bug #307818 */
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_ACCEPT);
 	
@@ -303,10 +299,7 @@ get_target_window (EphyHistoryWindow *editor)
 	}
 	else
 	{
-		EphySession *session;
-
-		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-		return GTK_WIDGET (ephy_session_get_active_window (session));
+		return GTK_WIDGET (gtk_application_get_active_window (GTK_APPLICATION (ephy_shell_get_default ())));
 	}
 }
 
@@ -324,8 +317,10 @@ cmd_open_bookmarks_in_tabs (GtkAction *action,
 	for (l = selection; l; l = l->next)
 	{
 		EphyHistoryURL *url = l->data;
-		ephy_shell_new_tab (ephy_shell, window, NULL, url->url,
-			EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+		ephy_shell_new_tab (ephy_shell_get_default (),
+				    window, NULL, url->url,
+				    EPHY_NEW_TAB_OPEN_PAGE |
+				    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	}
 
 	g_list_free_full (selection, (GDestroyNotify) ephy_history_url_free);
@@ -345,7 +340,8 @@ cmd_open_bookmarks_in_browser (GtkAction *action,
 	for (l = selection; l; l = l->next)
 	{
 		EphyHistoryURL *url = l->data;
-		ephy_shell_new_tab (ephy_shell, window, NULL, url->url,
+		ephy_shell_new_tab (ephy_shell_get_default (),
+				    window, NULL, url->url,
 				    EPHY_NEW_TAB_OPEN_PAGE |
 				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
@@ -460,13 +456,13 @@ cmd_delete (GtkAction *action,
 	{
 		GList *selected;
 		selected = ephy_urls_view_get_selection (EPHY_URLS_VIEW (editor->priv->pages_view));
-		ephy_history_service_delete_urls (editor->priv->history_service, selected, NULL,
+		ephy_history_service_delete_urls (editor->priv->history_service, selected, editor->priv->cancellable,
 						  (EphyHistoryJobCallback)on_browse_history_deleted_cb, editor);
 	} else if (gtk_widget_is_focus (editor->priv->hosts_view)) {
 		EphyHistoryHost *host = get_selected_host (editor);
 		if (host) {
 			ephy_history_service_delete_host (editor->priv->history_service,
-							  host, NULL,
+							  host, editor->priv->cancellable,
 							  (EphyHistoryJobCallback)on_host_deleted_cb,
 							  editor);
 			ephy_history_host_free (host);
@@ -498,7 +494,7 @@ static void
 cmd_help_contents (GtkAction *action,
 		   EphyHistoryWindow *editor)
 {
-	ephy_gui_help (GTK_WIDGET (editor), "ephy-managing-history");
+	ephy_gui_help (GTK_WIDGET (editor), "history");
 }
 
 G_DEFINE_TYPE (EphyHistoryWindow, ephy_history_window, GTK_TYPE_WINDOW)
@@ -570,7 +566,8 @@ ephy_history_window_row_activated_cb (GtkTreeView *view,
 						 path);
 	g_return_if_fail (url != NULL);
 
-	ephy_shell_new_tab (ephy_shell, NULL, NULL, url->url,
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    NULL, NULL, url->url,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 	ephy_history_url_free (url);
 }
@@ -587,7 +584,8 @@ ephy_history_window_row_middle_clicked_cb (EphyHistoryView *view,
 	url = ephy_urls_store_get_url_from_path (editor->priv->urls_store, path);
 	g_return_if_fail (url != NULL);
 
-	ephy_shell_new_tab (ephy_shell, window, NULL, url->url,
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    window, NULL, url->url,
 			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	ephy_history_url_free (url);
@@ -766,7 +764,7 @@ key_pressed_cb (GtkWidget *view,
 }
 
 static void
-search_entry_search_cb (GtkWidget *entry, char *search_text, EphyHistoryWindow *editor)
+search_entry_changed_cb (GtkWidget *entry, EphyHistoryWindow *editor)
 {
 	filter_now (editor, FALSE, TRUE);
 }
@@ -780,7 +778,7 @@ time_combo_changed_cb (GtkWidget *combo, EphyHistoryWindow *editor)
 static GtkWidget *
 build_search_box (EphyHistoryWindow *editor)
 {
-	GtkWidget *box, *label, *entry;
+	GtkWidget *box, *entry;
 	GtkWidget *combo;
 	char *str;
 
@@ -788,21 +786,13 @@ build_search_box (EphyHistoryWindow *editor)
 	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 	gtk_widget_show (box);
 
-	entry = ephy_search_entry_new ();
+	entry = gtk_search_entry_new ();
 	add_focus_monitor (editor, entry);
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
     
 	gtk_widget_show_all (entry);
 
-	label = gtk_label_new (NULL);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	str = g_strconcat ("<b>", _("_Search:"), "</b>", NULL);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
-	g_free (str);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-	gtk_widget_show (label);
-
 	combo = gtk_combo_box_text_new ();
 	gtk_widget_show (combo);
 
@@ -832,8 +822,6 @@ build_search_box (EphyHistoryWindow *editor)
 
 	editor->priv->time_combo = combo;
 
-	gtk_box_pack_start (GTK_BOX (box),
-			    label, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (box),
 			    entry, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (box),
@@ -842,8 +830,8 @@ build_search_box (EphyHistoryWindow *editor)
 	g_signal_connect (combo, "changed",
 			  G_CALLBACK (time_combo_changed_cb),
 			  editor);
-	g_signal_connect (G_OBJECT (entry), "search",
-			  G_CALLBACK (search_entry_search_cb),
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (search_entry_changed_cb),
 			  editor);
 
 	return box;
@@ -881,9 +869,13 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
         {
 		/* No need to use the async version as this function will be
 		called many times by the treeview. */
+#ifdef HAVE_WEBKIT2
+		/* TODO: Favicons */
+#else
 		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
 		pixbuf = webkit_favicon_database_get_favicon_pixbuf (database, page_location,
 								     FAVICON_SIZE, FAVICON_SIZE);
+#endif
         }
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
@@ -1076,7 +1068,7 @@ filter_now (EphyHistoryWindow *editor,
 	if (hosts)
 	{
 		ephy_history_service_find_hosts (editor->priv->history_service,
-						 from, to, NULL,
+						 from, to, editor->priv->cancellable,
 						(EphyHistoryJobCallback) on_get_hosts_cb, editor);
 	}
 
@@ -1086,17 +1078,15 @@ filter_now (EphyHistoryWindow *editor,
 		ephy_history_service_find_urls (editor->priv->history_service,
 						from, to,
 						0, host ? host->id : 0,
-						substrings, NULL,
+						substrings, editor->priv->cancellable,
 						(EphyHistoryJobCallback)on_find_urls_cb, editor);
 		ephy_history_host_free (host);
 	}
 }
 
 static gboolean
-on_visit_url_cb (EphyHistoryService *service,
-		 gchar *url,
-		 EphyHistoryPageVisitType visit_type,
-		 EphyHistoryWindow *editor)
+on_urls_visited_cb (EphyHistoryService *service,
+		    EphyHistoryWindow *editor)
 {
 	filter_now (editor, TRUE, TRUE);
 
@@ -1145,9 +1135,9 @@ ephy_history_window_constructed (GObject *object)
 
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
-	gtk_ui_manager_add_ui_from_file (ui_merge,
-					 ephy_file ("epiphany-history-window-ui.xml"),
-					 NULL);
+	gtk_ui_manager_add_ui_from_resource (ui_merge,
+					     "/org/gnome/epiphany/epiphany-history-window-ui.xml",
+					     NULL);
 	gtk_ui_manager_ensure_update (ui_merge);
 	editor->priv->ui_merge = ui_merge;
 	editor->priv->action_group = action_group;
@@ -1280,18 +1270,19 @@ ephy_history_window_constructed (GObject *object)
 			  G_CALLBACK (key_pressed_cb),
 			  editor);
 
-	ephy_state_add_window (GTK_WIDGET (editor),
-			       "history_window",
-			       450, 400, FALSE,
-			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
-	ephy_state_add_paned  (GTK_WIDGET (hpaned),
-			       "history_paned",
-			       130);
+	ephy_initial_state_add_window (GTK_WIDGET (editor),
+				       "history_window",
+				       450, 400, FALSE,
+				       EPHY_INITIAL_STATE_WINDOW_SAVE_SIZE | EPHY_INITIAL_STATE_WINDOW_SAVE_POSITION);
+	ephy_initial_state_add_paned  (GTK_WIDGET (hpaned),
+				       "history_paned",
+				       130);
 
+	editor->priv->cancellable = g_cancellable_new ();
 	filter_now (editor, TRUE, TRUE);
 
 	g_signal_connect_after (editor->priv->history_service,
-				"visit-url", G_CALLBACK (on_visit_url_cb),
+				"urls-visited", G_CALLBACK (on_urls_visited_cb),
 				editor);
 
 	if (G_OBJECT_CLASS (ephy_history_window_parent_class)->constructed)
@@ -1392,5 +1383,11 @@ ephy_history_window_dispose (GObject *object)
 		editor->priv->hosts_view = NULL;
 	}
 
+	if (editor->priv->cancellable)
+	{
+		g_cancellable_cancel (editor->priv->cancellable);
+		g_clear_object (&editor->priv->cancellable);
+	}
+
 	G_OBJECT_CLASS (ephy_history_window_parent_class)->dispose (object);
 }
