@@ -2,7 +2,7 @@
 /*
  *  Copyright © 2000-2002 Marco Pesenti Gritti
  *  Copyright © 2006, 2008 Christian Persch
- *  Copyright © 2011 Igalia S.L.
+ *  Copyright © 2011, 2012 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,36 +23,29 @@
 #include "config.h"
 
 #include "ephy-debug.h"
-#include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
+#include "ephy-initial-state.h"
 #include "ephy-private.h"
 #include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "ephy-state.h"
 #include "ephy-string.h"
 #include "ephy-web-app-utils.h"
 
 #include <errno.h>
-#include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libnotify/notify.h>
 #include <libxml/xmlversion.h>
 #include <string.h>
 
-#ifdef ENABLE_INTROSPECTION
-#include <girepository.h>
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
 #endif
 
-static GQuark startup_error_quark = 0;
-#define STARTUP_ERROR_QUARK (startup_error_quark)
-
 static gboolean open_in_new_tab = FALSE;
 static gboolean open_in_new_window = FALSE;
-static gboolean open_as_bookmarks_editor = FALSE;
-/*static gboolean reload_plugins = FALSE;*/
 
 static char *session_filename = NULL;
 static char *bookmark_url = NULL;
@@ -60,10 +53,10 @@ static char *bookmarks_file = NULL;
 static char **arguments = NULL;
 static char *application_to_delete = NULL;
 
-/* Only set from options in debug builds */
 static gboolean private_instance = FALSE;
+static gboolean incognito_mode = FALSE;
 static gboolean application_mode = FALSE;
-static gboolean keep_temp_directory = FALSE;
+static gboolean headless_mode = FALSE;
 static char *profile_directory = NULL;
 
 static gboolean
@@ -84,8 +77,6 @@ static const GOptionEntry option_entries[] =
     N_("Open a new tab in an existing browser window"), NULL },
   { "new-window", 0, 0, G_OPTION_ARG_NONE, &open_in_new_window,
     N_("Open a new browser window"), NULL },
-  { "bookmarks-editor", 'b', 0, G_OPTION_ARG_NONE, &open_as_bookmarks_editor,
-    N_("Launch the bookmarks editor"), NULL },
   { "import-bookmarks", '\0', 0, G_OPTION_ARG_FILENAME, &bookmarks_file,
     N_("Import bookmarks from the given file"), N_("FILE") },
   { "load-session", 'l', 0, G_OPTION_ARG_FILENAME, &session_filename,
@@ -94,8 +85,14 @@ static const GOptionEntry option_entries[] =
     N_("Add a bookmark"), N_("URL") },
   { "private-instance", 'p', 0, G_OPTION_ARG_NONE, &private_instance,
     N_("Start a private instance"), NULL },
+  { "incognito-mode", 'i', 0, G_OPTION_ARG_NONE, &incognito_mode,
+    N_("Start an instance in incognito mode"), NULL },
+  { "netbank-mode", 0, 0, G_OPTION_ARG_NONE, &incognito_mode,
+    N_("Start an instance in netbank mode"), NULL },
   { "application-mode", 'a', 0, G_OPTION_ARG_NONE, &application_mode,
     N_("Start the browser in application mode"), NULL },
+  { "headless-mode", 0, 0, G_OPTION_ARG_NONE, &headless_mode,
+    N_("Start the application without opening windows"), NULL },
   { "profile", 0, 0, G_OPTION_ARG_STRING, &profile_directory,
     N_("Profile directory to use in the private instance"), N_("DIR") },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &arguments,
@@ -107,15 +104,6 @@ static const GOptionEntry option_entries[] =
   { NULL }
 };
 
-#ifdef GNOME_ENABLE_DEBUG
-static GOptionEntry debug_option_entries[] =
-{
-  { "keep-tempdir", 0, 0, G_OPTION_ARG_NONE, &keep_temp_directory,
-    "Don't delete the temporary directory on exit", NULL },
-  { NULL }
-};
-#endif /* GNOME_ENABLE_DEBUG */
- 
 /* adapted from gtk+/gdk/x11/gdkdisplay-x11.c */
 static guint32
 get_startup_id (void)
@@ -146,6 +134,11 @@ get_startup_id (void)
   return retval;
 }
 
+#ifdef GDK_WINDOWING_X11
+/*
+ * FIXME: Need a solution for windowing-systems other than X11.
+ */
+
 /* Copied from libnautilus/nautilus-program-choosing.c; Needed in case
  * we have no DESKTOP_STARTUP_ID (with its accompanying timestamp).
  */
@@ -196,6 +189,7 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
   
   return event.xproperty.time;
 }
+#endif
 
 static void
 show_error_message (GError **error)
@@ -223,12 +217,13 @@ get_startup_flags (void)
 {
   EphyStartupFlags flags = 0;
 
+  if (headless_mode)
+    return EPHY_STARTUP_OPEN_NOTHING;
+
   if (open_in_new_tab)
     flags |= EPHY_STARTUP_NEW_TAB;
   if (open_in_new_window)
     flags |= EPHY_STARTUP_NEW_WINDOW;
-  if (open_as_bookmarks_editor)
-    flags |= EPHY_STARTUP_BOOKMARKS_EDITOR;
 
   return flags;
 }
@@ -245,6 +240,7 @@ main (int argc,
   EphyShellStartupContext *ctx;
   EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
+  EphyShell *ephy_shell;
   int status;
   EphyFileHelpersFlags flags;
 
@@ -331,21 +327,8 @@ main (int argc,
 
   g_option_context_set_main_group (option_context, option_group);
 
-#ifdef ENABLE_INTROSPECTION
-  g_option_context_add_group (option_context, g_irepository_get_option_group ());
-#endif
-
   g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
 
-#ifdef GNOME_ENABLE_DEBUG
-  option_group = g_option_group_new ("debug",
-                                     "Epiphany debug options",
-                                     "Epiphany debug options",
-                                     NULL, NULL);
-  g_option_group_add_entries (option_group, debug_option_entries);
-  g_option_context_add_group (option_context, option_group);
-#endif /* GNOME_ENABLE_DEBUG */
-
   if (!g_option_context_parse (option_context, &argc, &argv, &error)) {
     g_print ("Failed to parse arguments: %s\n", error->message);
     g_error_free (error);
@@ -361,21 +344,11 @@ main (int argc,
     exit (1);
   }
 
-  if (arguments != NULL && (session_filename != NULL || open_as_bookmarks_editor)) {
-    g_print ("Cannot use --bookmarks-editor or --load-session with URL arguments  \n");
-    exit (1);
-  }
-
   if (private_instance == TRUE && application_mode == TRUE) {
     g_print ("Cannot use --private-instance and --application-mode at the same time\n");
     exit (1);
   }
 
-  if (profile_directory != NULL && private_instance == FALSE && application_mode == FALSE) {
-    g_print ("--profile can only be used in combination with --private-instance or --application-mode\n");
-    exit (1);
-  }
-
   if (application_mode && profile_directory == NULL) {
     g_print ("--profile must be used when --application-mode is requested\n");
     exit (1);
@@ -386,12 +359,15 @@ main (int argc,
       exit (1);
   }
 
-  /* Work-around Flash Player crash */
-  g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
+  if (incognito_mode && profile_directory == NULL) {
+    g_print ("--incognito-mode needs a --profile parameter to steal the data from\n");
+    exit (1);
+  }
 
   /* Run the migration in all cases, except when running a private
-     instance without a given profile directory. */
-  if (!(private_instance && profile_directory == FALSE)) {
+     instance without a given profile directory or running in
+     incognito mode. */
+  if (!(private_instance && profile_directory == NULL) && incognito_mode == FALSE) {
     /* If the migration fails we don't really want to continue. */
     if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
       g_print ("Failed to run the migrator process, Web will now abort.");
@@ -402,9 +378,11 @@ main (int argc,
   /* Start our services */
   flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
 
-  if (private_instance || application_mode)
+  if (incognito_mode || private_instance || application_mode)
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
-  if (keep_temp_directory || profile_directory)
+  if (incognito_mode)
+    flags |= EPHY_FILE_HELPERS_STEAL_DATA;
+  if (profile_directory && !incognito_mode)
     flags |= EPHY_FILE_HELPERS_KEEP_DIR;
 
   if (!ephy_file_helpers_init (profile_directory, flags,
@@ -435,11 +413,16 @@ main (int argc,
     arguments = args;
   }
 
+#ifdef GDK_WINDOWING_X11
   /* Get a timestamp manually if need be */
-  if (user_time == 0)
-      user_time = slowly_and_stupidly_obtain_timestamp (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-
-  startup_error_quark = g_quark_from_static_string ("epiphany-startup-error");
+  if (user_time == 0) {
+    GdkDisplay* display =
+      gdk_display_manager_get_default_display (gdk_display_manager_get ());
+    if (GDK_IS_X11_DISPLAY (display))
+      user_time =
+        slowly_and_stupidly_obtain_timestamp (GDK_DISPLAY_XDISPLAY (display));
+  }
+#endif
 
   /* Delete the requested web application, if any. Must happen after
    * ephy_file_helpers_init (). */
@@ -451,7 +434,14 @@ main (int argc,
   /* Now create the shell */
   if (private_instance)
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
-  else if (application_mode) {
+  else if (incognito_mode) {
+    mode = EPHY_EMBED_SHELL_MODE_INCOGNITO;
+
+    /* Use the right theming. */
+    g_object_set (gtk_settings_get_default (),
+                  "gtk-application-prefer-dark-theme", TRUE,
+                  NULL);
+  } else if (application_mode) {
     char *app_name;
     char *app_icon;
 
@@ -475,6 +465,12 @@ main (int argc,
     }
 
     g_free (app_icon);
+  } else if (profile_directory) {
+    /* This mode exists purely for letting EphyShell know it should
+     * not consider this instance part of the unique application
+     * represented by the BROWSER mode.
+     */
+    mode = EPHY_EMBED_SHELL_MODE_STANDALONE;
   } else {
     mode = EPHY_EMBED_SHELL_MODE_BROWSER;
 
@@ -484,7 +480,6 @@ main (int argc,
     gtk_window_set_default_icon_name ("web-browser");
   }
 
-  ephy_embed_prefs_init ();
   _ephy_shell_create_instance (mode);
 
   startup_flags = get_startup_flags ();
@@ -495,6 +490,7 @@ main (int argc,
                                         arguments,
                                         user_time);
   g_strfreev (arguments);
+  ephy_shell = ephy_shell_get_default ();
   ephy_shell_set_startup_context (ephy_shell, ctx);
   status = g_application_run (G_APPLICATION (ephy_shell), argc, argv);
 
@@ -504,8 +500,7 @@ main (int argc,
   if (notify_is_initted ())
     notify_uninit ();
 
-  ephy_state_save ();
-  ephy_embed_prefs_shutdown ();
+  ephy_initial_state_save ();
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
