@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8 ts=8 sts=8 noet: */
 /*
  *  Copyright © 2000-2004 Marco Pesenti Gritti
  *  Copyright © 2009 Collabora Ltd.
@@ -30,9 +31,9 @@
 #include "ephy-embed-container.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
-#include "ephy-embed-single.h"
 #include "ephy-embed-utils.h"
 #include "ephy-embed.h"
+#include "ephy-favicon-helpers.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-find-toolbar.h"
@@ -46,9 +47,9 @@
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "ephy-state.h"
 #include "ephy-string.h"
 #include "ephy-web-app-utils.h"
+#include "ephy-web-dom-utils.h"
 #include "ephy-zoom.h"
 #include "pdm-dialog.h"
 
@@ -65,6 +66,9 @@
 #include <webkit/webkit.h>
 #endif
 
+#define DEFAULT_ICON_SIZE 144
+#define FAVICON_SIZE 16
+
 void
 window_cmd_file_print (GtkAction *action,
 		       EphyWindow *window)
@@ -80,6 +84,13 @@ window_cmd_file_print (GtkAction *action,
 	ephy_web_view_print (view);
 }
 
+void
+window_cmd_undo_close_tab (GtkAction *action,
+			   EphyWindow *window)
+{
+	ephy_session_undo_close_tab (ephy_shell_get_session (ephy_shell_get_default ()));
+}
+
 void
 window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
@@ -368,8 +379,10 @@ typedef struct {
 	GtkWidget *image;
 	GtkWidget *entry;
 	GtkWidget *spinner;
+	GtkWidget *spinner_box;
 	GtkWidget *box;
 	char *icon_href;
+	GdkRGBA icon_rgba;
 } EphyApplicationDialogData;
 
 static void
@@ -380,18 +393,168 @@ ephy_application_dialog_data_free (EphyApplicationDialogData *data)
 }
 
 static void
-take_page_snapshot_and_set_image (EphyApplicationDialogData *data)
+rounded_rectangle (cairo_t *cr,
+                   gdouble  aspect,
+                   gdouble  x,
+                   gdouble  y,
+                   gdouble  corner_radius,
+                   gdouble  width,
+                   gdouble  height)
 {
-	GdkPixbuf *snapshot;
-	int x, y, w, h;
+        gdouble radius;
+        gdouble degrees;
 
-	x = y = 0;
-	w = h = 128; /* GNOME hi-res icon size. */
+        radius = corner_radius / aspect;
+        degrees = G_PI / 180.0;
 
-	snapshot = ephy_web_view_get_snapshot (data->view, x, y, w, h);
+        cairo_new_sub_path (cr);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + radius,
+                   radius,
+                   -90 * degrees,
+                   0 * degrees);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + height - radius,
+                   radius,
+                   0 * degrees,
+                   90 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + height - radius,
+                   radius,
+                   90 * degrees,
+                   180 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + radius,
+                   radius,
+                   180 * degrees,
+                   270 * degrees);
+        cairo_close_path (cr);
+}
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), snapshot);
-	g_object_unref (snapshot);
+static GdkPixbuf *
+frame_pixbuf (GdkPixbuf  *pixbuf,
+	      GdkRGBA    *rgba,
+	      int         width,
+	      int         height)
+{
+	GdkPixbuf *framed;
+	cairo_surface_t *surface;
+	cairo_t *cr;
+	int frame_width;
+	int radius;
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					      width, height);
+	cr = cairo_create (surface);
+
+	frame_width = 0;
+	radius = 20;
+
+	rounded_rectangle (cr,
+			   1.0,
+			   frame_width + 0.5,
+			   frame_width + 0.5,
+			   radius,
+			   width - frame_width * 2 - 1,
+			   height - frame_width * 2 - 1);
+	if (rgba != NULL)
+			cairo_set_source_rgba (cr,
+					       rgba->red,
+					       rgba->green,
+					       rgba->blue,
+					       rgba->alpha);
+	else
+		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.3);
+	cairo_fill_preserve (cr);
+
+	if (pixbuf != NULL) {
+		GdkPixbuf *scaled;
+		int w;
+		int h;
+
+		w = gdk_pixbuf_get_width (pixbuf);
+		h = gdk_pixbuf_get_height (pixbuf);
+
+		if (w < 48 || h < 48) {
+			scaled = gdk_pixbuf_scale_simple (pixbuf, w * 3, h * 3, GDK_INTERP_NEAREST);
+		} else if (w > width || h > height) {
+			double ws, hs, s;
+
+			ws = (double) width / w;
+			hs = (double) height / h;
+			s = MIN (ws, hs);
+			scaled = gdk_pixbuf_scale_simple (pixbuf, w * s, h * s, GDK_INTERP_BILINEAR);
+		} else {
+			scaled = g_object_ref (pixbuf);
+		}
+
+		w = gdk_pixbuf_get_width (scaled);
+		h = gdk_pixbuf_get_height (scaled);
+
+		gdk_cairo_set_source_pixbuf (cr, scaled,
+					     (width - w) / 2,
+					     (height - h) / 2);
+		g_object_unref (scaled);
+		cairo_fill (cr);
+	}
+
+	framed = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+	cairo_destroy (cr);
+	cairo_surface_destroy (surface);
+
+	return framed;
+}
+
+static void
+set_image_from_favicon (EphyApplicationDialogData *data)
+{
+	GdkPixbuf *icon = NULL;
+
+#ifdef HAVE_WEBKIT2
+	{
+		cairo_surface_t *icon_surface = webkit_web_view_get_favicon (WEBKIT_WEB_VIEW (data->view));
+		if (icon_surface)
+			icon = ephy_pixbuf_get_from_surface_scaled (icon_surface, 0, 0);
+	}
+#else
+	{
+		const char *page_uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view));
+		if (page_uri)
+			icon = webkit_favicon_database_try_get_favicon_pixbuf (webkit_get_favicon_database (),
+									       page_uri,
+									       FAVICON_SIZE, FAVICON_SIZE);
+	}
+#endif
+
+	if (icon != NULL) {
+		GdkPixbuf *framed;
+
+		framed = frame_pixbuf (icon, NULL, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+		g_object_unref (icon);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), framed);
+		g_object_unref (framed);
+	}
+}
+
+static void
+set_app_icon_from_filename (EphyApplicationDialogData *data,
+			    const char *filename)
+{
+	GdkPixbuf *pixbuf;
+	GdkPixbuf *framed;
+
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	if (pixbuf == NULL)
+		return;
+
+	framed = frame_pixbuf (pixbuf, &data->icon_rgba, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	g_object_unref (pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), framed);
+	g_object_unref (framed);
 }
 
 #ifdef HAVE_WEBKIT2
@@ -401,8 +564,10 @@ download_finished_cb (WebKitDownload *download,
 {
 	char *filename;
 
+	gtk_widget_show (data->image);
+
 	filename = g_filename_from_uri (webkit_download_get_destination (download), NULL, NULL);
-	gtk_image_set_from_file (GTK_IMAGE (data->image), filename);
+	set_app_icon_from_filename (data, filename);
 	g_free (filename);
 }
 
@@ -411,9 +576,11 @@ download_failed_cb (WebKitDownload *download,
 		    GError *error,
 		    EphyApplicationDialogData *data)
 {
+	gtk_widget_show (data->image);
+
 	g_signal_handlers_disconnect_by_func (download, download_finished_cb, data);
 	/* Something happened, default to a page snapshot. */
-	take_page_snapshot_and_set_image (data);
+	set_image_from_favicon (data);
 }
 #else
 static void
@@ -424,18 +591,20 @@ download_status_changed_cb (WebKitDownload *download,
 	WebKitDownloadStatus status = webkit_download_get_status (download);
 	char *filename;
 
+	gtk_widget_show (data->image);
+
 	switch (status)
 	{
 	case WEBKIT_DOWNLOAD_STATUS_FINISHED:
 		filename = g_filename_from_uri (webkit_download_get_destination_uri (download),
 						   NULL, NULL);
-		gtk_image_set_from_file (GTK_IMAGE (data->image), filename);
+		set_app_icon_from_filename (data, filename);
 		g_free (filename);
 		break;
 	case WEBKIT_DOWNLOAD_STATUS_ERROR:
 	case WEBKIT_DOWNLOAD_STATUS_CANCELLED:
 		/* Something happened, default to a page snapshot. */
-		take_page_snapshot_and_set_image (data);
+		set_image_from_favicon (data);
 		break;
 	default:
 		break;
@@ -455,6 +624,12 @@ download_icon_and_set_image (EphyApplicationDialogData *data)
 #ifdef HAVE_WEBKIT2
 	download = webkit_web_context_download_uri (webkit_web_context_get_default (),
 						    data->icon_href);
+	/* We do not want this download to show up in the UI, so let's
+	 * set 'ephy-download-set' to make Epiphany think this is
+	 * already there. */
+	/* FIXME: it's probably better to just do this in a clean way
+	 * instead of using this workaround. */
+	g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
 #else
 	request = webkit_network_request_new (data->icon_href);
 	download = webkit_download_new (request);
@@ -486,46 +661,213 @@ download_icon_and_set_image (EphyApplicationDialogData *data)
 #endif
 }
 
+static void
+download_icon_or_take_snapshot (EphyApplicationDialogData *data,
+				gboolean res,
+				char *uri,
+				char *color)
+{
+	if (uri != NULL && uri[0] != '\0')
+		data->icon_href = uri;
+
+	if (color != NULL && color[0] != '\0')
+	{
+		gdk_rgba_parse (&data->icon_rgba, color);
+	}
+	else
+	{
+		data->icon_rgba.red = 0.5;
+		data->icon_rgba.green = 0.5;
+		data->icon_rgba.blue = 0.5;
+		data->icon_rgba.alpha = 0.3;
+	}
+
+	if (res)
+	{
+		download_icon_and_set_image (data);
+	}
+	else
+	{
+		gtk_widget_show (data->image);
+		set_image_from_favicon (data);
+	}
+}
+
+#ifdef HAVE_WEBKIT2
+static void
+fill_default_application_image_cb (GObject *source,
+				   GAsyncResult *async_result,
+				   gpointer user_data)
+{
+	EphyApplicationDialogData *data = user_data;
+	GVariant *result;
+	char *uri = NULL;
+	char *color = NULL;
+	gboolean res = FALSE;
+
+	result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source),
+					   async_result,
+					   NULL);
+
+	if (result)
+	{
+		g_variant_get (result, "(bss)", &res, &uri, &color);
+		g_variant_unref (result);
+	}
+
+	download_icon_or_take_snapshot (data, res, uri, color);
+}
+#endif
+
 static void
 fill_default_application_image (EphyApplicationDialogData *data)
 {
+	const char *base_uri;
 #ifdef HAVE_WEBKIT2
-	/* TODO: DOM Bindindgs */
+	GDBusProxy *web_extension;
 #else
-	WebKitDOMDocument *document;
-	WebKitDOMNodeList *links;
-	gulong length, i;
+	char *uri = NULL;
+	char *color = NULL;
+	gboolean res;
+#endif
 
-	document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (data->view));
-	links = webkit_dom_document_get_elements_by_tag_name (document, "link");
-	length = webkit_dom_node_list_get_length (links);
+	base_uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view));
 
-	for (i = 0; i < length; i++)
+#ifdef HAVE_WEBKIT2
+	web_extension = ephy_embed_shell_get_web_extension_proxy (ephy_embed_shell_get_default ());
+	if (web_extension)
+		g_dbus_proxy_call (web_extension,
+				   "GetBestWebAppIcon",
+				   g_variant_new("(ts)", webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (data->view)), base_uri),
+				   G_DBUS_CALL_FLAGS_NONE,
+				   -1,
+				   NULL,
+				   fill_default_application_image_cb,
+				   data);
+	else
+		download_icon_or_take_snapshot (data, FALSE, NULL, NULL);
+#else
+	res = ephy_web_dom_utils_get_best_icon (webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (data->view)),
+						base_uri,
+						&uri,
+						&color);
+
+	download_icon_or_take_snapshot (data, res, uri, color);
+#endif
+}
+
+typedef struct {
+	const char *host;
+	const char *name;
+} SiteInfo;
+
+static SiteInfo sites[] = {
+	{ "www.facebook.com", "Facebook" },
+	{ "twitter.com", "Twitter" },
+	{ "gmail.com", "GMail" },
+	{ "plus.google.com", "Google+" },
+	{ "youtube.com", "YouTube" },
+};
+
+static char *
+get_special_case_application_title_for_host (const char *host)
+{
+	char *title = NULL;
+	int i;
+
+	for (i = 0; i < G_N_ELEMENTS (sites) && title == NULL; i++)
 	{
-		char *rel;
-		WebKitDOMNode *node = webkit_dom_node_list_item (links, i);
-		rel = webkit_dom_html_link_element_get_rel (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
-		/* TODO: support more than one possible icon. */
-		if (g_strcmp0 (rel, "apple-touch-icon") == 0 ||
-		    g_strcmp0 (rel, "apple-touch-icon-precomposed") == 0)
+		SiteInfo *info = &sites[i];
+		if (strcmp (host, info->host) == 0)
 		{
-			data->icon_href = webkit_dom_html_link_element_get_href (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
-			download_icon_and_set_image (data);
-			g_free (rel);
-			return;
+			title = g_strdup (info->name);
 		}
 	}
-#endif
-	/* If we make it here, no "apple-touch-icon" link was
-	 * found. Take a snapshot of the page. */
-	take_page_snapshot_and_set_image (data);
+
+	return title;
 }
 
+static void
+set_default_application_title (EphyApplicationDialogData *data,
+			       char *title)
+{
+	if (title == NULL || title[0] == '\0')
+	{
+		SoupURI *uri;
+		const char *host;
+
+		uri = soup_uri_new (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)));
+		host = soup_uri_get_host (uri);
+
+		if (host != NULL && host[0] != '\0')
+			title = get_special_case_application_title_for_host (host);
+
+		if (title == NULL || title[0] == '\0')
+		{
+			if (g_str_has_prefix (host, "www."))
+				title = g_strdup (host + strlen ("www."));
+			else
+				title = g_strdup (host);
+		}
+
+		soup_uri_free (uri);
+	}
+
+	if (title == NULL || title[0] == '\0')
+	{
+		title = g_strdup (ephy_web_view_get_title (data->view));
+	}
+
+	gtk_entry_set_text (GTK_ENTRY (data->entry), title);
+	g_free (title);
+}
+
+#ifdef HAVE_WEBKIT2
+static void
+fill_default_application_title_cb (GObject *source,
+				   GAsyncResult *async_result,
+				   gpointer user_data)
+{
+	EphyApplicationDialogData *data = user_data;
+	GVariant *result;
+	char *title = NULL;
+
+	result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source),
+					   async_result,
+					   NULL);
+
+	if (result)
+	{
+		g_variant_get (result, "(s)", &title);
+		g_variant_unref (result);
+	}
+
+	set_default_application_title (data, title);
+}
+#endif
+
 static void
 fill_default_application_title (EphyApplicationDialogData *data)
 {
-	const char *title = ephy_web_view_get_title (data->view);
-	gtk_entry_set_text (GTK_ENTRY (data->entry), title);
+#ifdef HAVE_WEBKIT2
+	GDBusProxy *web_extension;
+	web_extension = ephy_embed_shell_get_web_extension_proxy (ephy_embed_shell_get_default ());
+	if (web_extension)
+		g_dbus_proxy_call (web_extension,
+				   "GetWebAppTitle",
+				   g_variant_new("(t)", webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (data->view))),
+				   G_DBUS_CALL_FLAGS_NONE,
+				   -1,
+				   NULL,
+				   fill_default_application_title_cb,
+				   data);
+	else
+		set_default_application_title (data, NULL);
+#else
+	WebKitDOMDocument *document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (data->view));
+	char *title = ephy_web_dom_utils_get_application_title (document);
+	set_default_application_title (data, title);
+#endif
 }
 
 static void
@@ -534,10 +876,7 @@ notify_launch_cb (NotifyNotification *notification,
 		  gpointer user_data)
 {
 	char * desktop_file = user_data;
-	/* A gross hack to be able to launch epiphany from within
-	 * Epiphany. Might be a good idea to figure out a better
-	 * solution... */
-	g_unsetenv (EPHY_UUID_ENVVAR);
+
 	ephy_file_launch_desktop_file (desktop_file, NULL, 0, NULL);
 	g_free (desktop_file);
 }
@@ -617,6 +956,7 @@ dialog_save_as_application_response_cb (GtkDialog *dialog,
 
 		notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
 		notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
+		notify_notification_set_hint (notification, "desktop-entry", g_variant_new_string ("epiphany"));
 		notify_notification_set_hint (notification, "transient", g_variant_new_boolean (TRUE));
 		notify_notification_show (notification, NULL);
 	}
@@ -631,8 +971,14 @@ window_cmd_file_save_as_application (GtkAction *action,
 {
 	EphyEmbed *embed;
 	GtkWidget *dialog, *box, *image, *entry, *content_area;
+	GtkWidget *label;
+	GtkWidget *spinner;
+	GtkWidget *alignment;
 	EphyWebView *view;
 	EphyApplicationDialogData *data;
+	GdkPixbuf *pixbuf;
+	GtkStyleContext *context;
+	char *markup;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
@@ -642,32 +988,61 @@ window_cmd_file_save_as_application (GtkAction *action,
 	/* Show dialog with icon, title. */
 	dialog = gtk_dialog_new_with_buttons (_("Create Web Application"),
 					      GTK_WINDOW (window),
-					      0,
-					      GTK_STOCK_CANCEL,
+					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					      _("_Cancel"),
 					      GTK_RESPONSE_CANCEL,
 					      _("C_reate"),
 					      GTK_RESPONSE_OK,
 					      NULL);
 
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 14 + 2 * 5 = 24 */
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
 
-	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
 	gtk_container_add (GTK_CONTAINER (content_area), box);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 5);
 
 	image = gtk_image_new ();
-	gtk_widget_set_size_request (image, 128, 128);
+	gtk_widget_set_no_show_all (image, TRUE);
+	gtk_widget_set_size_request (image, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_widget_set_margin_bottom (image, 10);
 	gtk_container_add (GTK_CONTAINER (box), image);
+	pixbuf = frame_pixbuf (NULL, NULL, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+	g_object_unref (pixbuf);
+
+	alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
+	gtk_widget_set_no_show_all (alignment, TRUE);
+	gtk_widget_set_size_request (alignment, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
+	gtk_container_add (GTK_CONTAINER (box), alignment);
+	gtk_widget_show (alignment);
+
+	spinner = gtk_spinner_new ();
+	gtk_widget_set_size_request (spinner, 22, 22);
+	gtk_spinner_start (GTK_SPINNER (spinner));
+	gtk_container_add (GTK_CONTAINER (alignment), spinner);
+	gtk_widget_show (spinner);
 
 	entry = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-	gtk_box_pack_end (GTK_BOX (box), entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0);
+
+	markup = g_strdup_printf ("<small>%s</small>", webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view)));
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label), markup);
+	g_free (markup);
+	gtk_box_pack_end (GTK_BOX (box), label, FALSE, FALSE, 0);
+	context = gtk_widget_get_style_context (label);
+	gtk_style_context_add_class (context, "dim-label");
 
 	data = g_slice_new0 (EphyApplicationDialogData);
 	data->view = view;
 	data->image = image;
 	data->entry = entry;
+	data->spinner = spinner;
+	data->spinner_box = alignment;
+
+	g_object_bind_property (image, "visible", data->spinner_box, "visible", G_BINDING_INVERT_BOOLEAN);
 
 	fill_default_application_image (data);
 	fill_default_application_title (data);
@@ -681,21 +1056,6 @@ window_cmd_file_save_as_application (GtkAction *action,
 	gtk_widget_show_all (dialog);
 }
 
-void
-window_cmd_file_work_offline (GtkAction *action,
-		              EphyWindow *window)
-{
-        /* TODO: WebKitGTK+ does not currently support offline status. */
-#if 0
-	EphyEmbedSingle *single;
-	gboolean offline;
-
-	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
-	offline = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	ephy_embed_single_set_network_status (single, !offline);
-#endif
-}
-
 void
 window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
@@ -722,7 +1082,7 @@ void
 window_cmd_file_quit (GtkAction *action,
 		      EphyWindow *window)
 {
-	if (ephy_session_close_all_windows (EPHY_SESSION (ephy_shell_get_session (ephy_shell))))
+	if (ephy_shell_close_all_windows (ephy_shell_get_default ()))
 		g_application_quit (g_application_get_default ());
 }
 
@@ -730,8 +1090,19 @@ void
 window_cmd_file_new_window (GtkAction *action,
 			    EphyWindow *window)
 {
-	ephy_shell_new_tab (ephy_shell, NULL, NULL, NULL,
-			    EPHY_NEW_TAB_IN_NEW_WINDOW | EPHY_NEW_TAB_HOME_PAGE);
+	ephy_shell_new_tab (ephy_shell_get_default (),
+			    NULL, NULL, NULL,
+			    EPHY_NEW_TAB_IN_NEW_WINDOW |
+			    EPHY_NEW_TAB_HOME_PAGE);
+}
+
+void
+window_cmd_file_new_incognito_window (GtkAction *action,
+				      EphyWindow *window)
+{
+	char *str = g_strdup_printf ("epiphany --incognito-mode --profile %s", ephy_dot_dir ());
+	g_spawn_command_line_async (str, NULL);
+	g_free (str);
 }
 
 void
@@ -952,7 +1323,7 @@ window_cmd_edit_bookmarks (GtkAction *action,
 {
 	GtkWidget *bwindow;
 	
-	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell);
+	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell_get_default ());
 	gtk_window_present (GTK_WINDOW (bwindow));
 }
 
@@ -962,7 +1333,7 @@ window_cmd_edit_history (GtkAction *action,
 {
 	GtkWidget *hwindow;
 	
-	hwindow = ephy_shell_get_history_window (ephy_shell);
+	hwindow = ephy_shell_get_history_window (ephy_shell_get_default ());
 	gtk_window_present (GTK_WINDOW (hwindow));
 }
 
@@ -972,7 +1343,7 @@ window_cmd_edit_preferences (GtkAction *action,
 {
 	EphyDialog *dialog;
 	
-	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell));
+	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell_get_default ()));
 	
 	ephy_dialog_show (dialog);
 }
@@ -983,7 +1354,7 @@ window_cmd_edit_personal_data (GtkAction *action,
 {
 	PdmDialog *dialog;
 	
-	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
+	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell_get_default ()));
 	/* FIXME?: pdm_dialog_open is supposed to scroll to the host passed
 	 * as second parameters in the cookies tab. Honestly I think this
 	 * has been broken for a while. In any case it's probably not
@@ -1035,13 +1406,14 @@ view_source_embedded (const char *uri, EphyEmbed *embed)
 			 NULL,
 			 EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_APPEND_AFTER);
 #ifdef HAVE_WEBKIT2
-	/* TODO: View Source */
+	webkit_web_view_set_view_mode
+		(EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed), WEBKIT_VIEW_MODE_SOURCE);
 #else
 	webkit_web_view_set_view_source_mode
 		(EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed), TRUE);
+#endif
 	webkit_web_view_load_uri
 		(EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed), uri);
-#endif
 }
 
 
@@ -1263,16 +1635,12 @@ window_cmd_view_page_source (GtkAction *action,
 
 	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
-#ifdef HAVE_WEBKIT2
-	/* TODO: View Source */
-#else
 	if (g_settings_get_boolean (EPHY_SETTINGS_MAIN,
 				    EPHY_PREFS_INTERNAL_VIEW_SOURCE))
 	{
 		view_source_embedded (address, embed);
 		return;
 	}
-#endif
 
 	user_time = gtk_get_current_event_time ();
 
@@ -1291,6 +1659,13 @@ window_cmd_view_page_source (GtkAction *action,
 	}
 }
 
+void
+window_cmd_help_contents (GtkAction *action,
+			  GtkWidget *window)
+{
+	ephy_gui_help (window, NULL);
+}
+
 #define ABOUT_GROUP "About"
 
 void
@@ -1412,7 +1787,7 @@ window_cmd_help_about (GtkAction *action,
 			       "program-name", _("Web"),
 			       "version", VERSION,
 			       "copyright", "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-			                    "Copyright © 2003–2012 The Web Developers",
+			                    "Copyright © 2003–2013 The Web Developers",
 			       "artists", artists,
 			       "authors", authors,
 			       "comments", comments,
@@ -1573,7 +1948,7 @@ window_cmd_browse_with_caret (GtkAction *action,
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 							  _("Pressing F7 turns caret browsing on or off. This feature "
 							    "places a moveable cursor in web pages, allowing you to move "
-							    "around with your keyboard. Do you want to enable caret browsing on?"));
+							    "around with your keyboard. Do you want to enable caret browsing?"));
 		gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Enable"), GTK_RESPONSE_ACCEPT);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
